/***************************************************************************
*
*   Module:             diags
*
*   Filename:           sysevent.c
*
*   Author:             Brian Tran
*
*   Created:            09/03/2002
*
*   Description:
*
*   This device is read-only from user perspective. The bloking read returns when there is an interrput on system events
*   such as fan faults, over temperature, power supply failure. A user library is needed to find out what exactly what when wrong.
*   This device only returns the one-byte cause code of the event.  
*
****************************************************************************
*                 Source Control System Information
*
*    $Id: bldsc_sysevent.c,v 1.7 2004/05/20 02:39:16 brian Exp $
*
****************************************************************************
*                       Modification History
*
*    $Log: bldsc_sysevent.c,v $
*    Revision 1.7  2004/05/20 02:39:16  brian
*    Bugzilla number:
*    Sysevent support for bldsc
*
*    Revision 1.6  2004/05/04 19:27:47  brian
*    Bugzilla number:
*    Init the last time the bus is disconnected to zero (make sure it is not negative).
*
*    Revision 1.5  2004/05/04 16:54:26  brian
*    Bugzilla number:
*    Support for i2c arbitration in bldsc
*
*    Revision 1.4  2004/04/23 23:49:48  brian
*    Bugzilla number:
*    Support for lod BC boards in which the i2c switch does not exist. Inthis case, the kernel will not return error, and apps will go on and access the devices.
*
*    Revision 1.3  2004/04/21 21:08:30  brian
*    Bugzilla number:
*    Support I2C debug
*
*    Revision 1.2  2004/04/10 02:15:52  brian
*    Bugzilla number:
*    Add support for i2c switch in bldsc
*
*    Revision 1.1  2003/12/19 00:28:04  andy
*    Blade Center Switch Card board specific kernel changes
*
*    Revision 1.2  2003/10/30 23:08:31  brian
*    Bugzilla number:
*    Support seti2croute in 120 kernel
*
*    Revision 1.1  2003/10/29 01:16:00  brian
*    Bugzilla number:
*    For 120
*
*
***************************************************************************
*                       (c) COPYRIGHT 2000, 2001, 2002
*                           TopSpin Communications, Inc.
*                           Palo Alto, CA
**************************************************************************/
#ifdef RCSID
static const char rcsid[] = "$Id: bldsc_sysevent.c,v 1.7 2004/05/20 02:39:16 brian Exp $";
#endif


#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/poll.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/sched.h>
#include <linux/i2c-errcode.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/machdep.h>
#include <asm/ocp.h>
#include <system_interrupt_code.h>
#include <fpga.h>
#include "bldsc_sysevent.h"
#include <linux/i2c-errcode.h>
#include <linux/slab.h>
#include "ds1672_rtc.h"
#include "dcr.h"

extern int global_i2c_debug;
extern int i2c_controller_fast_mode;
extern struct semaphore i2c_lock;


extern int phy_iic_xfer(void *vaddr, unsigned short dev_addr, int access, char *buf, int count, int controller);

int sysevent_init_module(void);
void sysevent_cleanup_module(void);
int seti2croute(void *vaddr, unsigned char muxselect, int (*iic_xfer)(void *, unsigned short, int, char *, int, int), int controller);
int is_bld_i2cbus1_disconnected(void);
int bld_i2cbus1_disconnect(void);
void bld_i2cbus1_connect(unsigned long dummy);
int bld_i2cbus1_connect_callback(void *arg);


/* Function Prototypes */
static ssize_t sysevent_read(struct file *file, char *buf, size_t count, loff_t *offset);
static ssize_t sysevent_write(struct file *file, const char *buf, size_t count, loff_t *offset);
static unsigned int sysevent_poll(struct file *file, poll_table *wait);
static int sysevent_ioctl (struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);



/* File Operation Table */
static struct file_operations char_fops = {
    owner: THIS_MODULE,
    read: sysevent_read,
    write: sysevent_write,
    ioctl: sysevent_ioctl,
    poll: sysevent_poll,
};

static wait_queue_head_t sysevent_waitq;
static wait_queue_head_t delay_waitq;
static wait_queue_head_t i2cbus1_waitq;

static struct tq_struct bld_i2cbus1_connect_callback_tq = {
    routine: bld_i2cbus1_connect_callback,
};


static volatile int fpga_int_active;
static volatile int pic_int_active;

#define I2C_CONTROLLER1_PHY_ADDR    0x140000500LL

static int timeout = 10;
static unsigned long volatile last_time_i2cbus1_disconnected;
static struct timer_list *reconnect_timer;

static void *fpga_base_addr;
static void *i2c_base_addr;


/* interrupt handler */
static void fpga_int_handler(int this_irq, void *dev_id, struct pt_regs *regs)
{
    fpga_int_active = 1;
    wake_up_interruptible(&sysevent_waitq);
}

static void pic_int_handler(int this_irq, void *dev_id, struct pt_regs *regs)
{
    pic_int_active = 1;
    wake_up_interruptible(&sysevent_waitq);
}


void bld_i2cbus1_connect(unsigned long dummy)
{
    bld_i2cbus1_connect_callback_tq.data = (void *) 0;
    schedule_task(&bld_i2cbus1_connect_callback_tq);

    return;
}


int bld_i2cbus1_connect_callback(void *arg)
{
    unsigned char data;

    /* make sure no more i2c transaction */
    down(&i2c_lock);

    data = readb((void *) ((int) fpga_base_addr + SC_BLD_FPGA_MISC_CNTL_REG0_OFFSET));
    data &= ~SC_BLD_FPGA_MISC_CNTL_I2C_CONNECT_MASK;
    writeb(data, (void *) ((int) fpga_base_addr + SC_BLD_FPGA_MISC_CNTL_REG0_OFFSET));
    udelay(100);

    up(&i2c_lock);

    /* remember the last time the bus is disconnected */ 
    last_time_i2cbus1_disconnected = jiffies;


    return;
}


int is_bld_i2cbus1_disconnected()
{
    unsigned char data;

    data = readb((void *) ((int) fpga_base_addr + SC_BLD_FPGA_MISC_CNTL_REG0_OFFSET));

    if ((data & SC_BLD_FPGA_MISC_CNTL_I2C_CONNECT_MASK) != 0) {     /* bit == 1 means disconnected */
        return 1;
    }

    return 0;
}


#define BLD_MAX_WAIT_CONNECT	25
#define BLD_MAX_I2CBUS1_DISCONNECT_TIME		(HZ/5)		/* 200 ms */
#define BLD_MIN_I2CBUS1_BETWEEN_DISCONNECT	(HZ/2)		/* 500 ms */

int bld_i2cbus1_disconnect()
{
    unsigned char data;
    volatile unsigned char status;
    int count;


    if (is_bld_i2cbus1_disconnected() == 1) {	/* already disconnected, do nothing */
        return 0;
    }

    /* bus is being connected. Make sure time has elapsed more than 1 sec since last time bus was disconnected */
    while ((jiffies - last_time_i2cbus1_disconnected) < BLD_MIN_I2CBUS1_BETWEEN_DISCONNECT) {
         interruptible_sleep_on_timeout(&i2cbus1_waitq, HZ/5);
    }

    /* waiting on i2c bus not free */
    count = 0;
    do {
        status = readb((void *) ((int) i2c_base_addr + I2C_CONTROLLER_EXT_STATUS));
        status = status & 0x70;		/* get the bus status */
        if (status == 0x40) {	/* bus free */
            break;
        }
        count ++;
        /* DEB(printk("Bus busy, wait ...\n")); */
        interruptible_sleep_on_timeout(&i2cbus1_waitq, HZ/100);		/* 10 ms */
        /* udelay(1000); */
    } while (count < BLD_MAX_WAIT_CONNECT);
            
    if (count == BLD_MAX_WAIT_CONNECT) {
        return IIC_ERR_BUS_BUSY;
    }

    /* actually disconnect the bus */ 
    data = readb((void *) ((int) fpga_base_addr + SC_BLD_FPGA_MISC_CNTL_REG0_OFFSET));
    data |= SC_BLD_FPGA_MISC_CNTL_I2C_CONNECT_MASK;
    writeb(data, (void *) ((int) fpga_base_addr + SC_BLD_FPGA_MISC_CNTL_REG0_OFFSET));
    udelay(100);


    /* start re-connect timer */
    reconnect_timer->expires = jiffies + BLD_MAX_I2CBUS1_DISCONNECT_TIME;
    add_timer(reconnect_timer);


    return 0;
}

    



int sysevent_init_module(void)
{
    int dcr_val;

    /* init global vars */
    fpga_int_active = 0;
    pic_int_active = 0;


    /* register char device */
    if (register_chrdev(SYSEVENT_MAJOR, "syseventmodule", &char_fops)) {
        printk(KERN_ERR "Unable to register syseventmodule\n");
        return -EIO;
    }

    fpga_base_addr = ioremap_nocache(FPGA_REGADDR_BASE, 0x100); 
    i2c_base_addr = ioremap_nocache(I2C_CONTROLLER1_PHY_ADDR, 0x12);

    /* init re-connect timer */
    reconnect_timer = kmalloc (sizeof (*reconnect_timer), GFP_KERNEL);
    if (!reconnect_timer) {
        return -ENOMEM;
    }
    init_timer(reconnect_timer);
    reconnect_timer->data = 0;
    reconnect_timer->function = bld_i2cbus1_connect;


    /* make sure the I2C bus 1 is connected to the chassis bus */
    bld_i2cbus1_connect(0);
    last_time_i2cbus1_disconnected = 0;	


    /* init wait queue */
    init_waitqueue_head(&sysevent_waitq);
    init_waitqueue_head(&delay_waitq);
    init_waitqueue_head(&i2cbus1_waitq);

    /* configure IRQ 1, 5 to be trigger type, falling edge */
    dcr_val = mfdcr(UIC0_TRIGGER_REG);
    dcr_val |= BLDSC_UIC0_INT_MASK;
    mtdcr(UIC0_TRIGGER_REG, dcr_val);

    dcr_val = mfdcr(UIC0_POLARITY_REG);
    dcr_val &= (~BLDSC_UIC0_INT_MASK);
    mtdcr(UIC0_POLARITY_REG, dcr_val);

    /* clear int status */
    dcr_val = mfdcr(UIC0_STATUS_REG);
    dcr_val |= BLDSC_UIC0_INT_MASK;
    mtdcr(UIC0_STATUS_REG, dcr_val);


    if (request_irq(FPGA_IRQ_NUM, fpga_int_handler, SA_INTERRUPT, "FPGA Int", NULL)) {
        printk(KERN_ERR "sysevent_init_module: Request IRQ %d Failed\n", FPGA_IRQ_NUM);
        return -1;
    }


    if (request_irq(PIC_IRQ_NUM, pic_int_handler, SA_INTERRUPT, "PIC Int", NULL)) {
        printk(KERN_ERR "sysevent_init_module: Request IRQ %d Failed\n", PIC_IRQ_NUM);
        return -1;
    }

    /* we clear and enable intr in the fpga and PIC in user land */

    return 0;
}


void sysevent_cleanup_module(void)
{

    unregister_chrdev(SYSEVENT_MAJOR, "syseventmodule");
    iounmap(fpga_base_addr);
    iounmap(i2c_base_addr);

    if (reconnect_timer) {
        del_timer(reconnect_timer);
        kfree(reconnect_timer);
    }

    free_irq(FPGA_IRQ_NUM, NULL);
    free_irq(PIC_IRQ_NUM, NULL);
}


/* Read routine */
static ssize_t sysevent_read(struct file *file, char *buf, size_t count, loff_t *offset)
{
    int interrupt_code;
    int delay;


    if ((fpga_int_active == 0) && (pic_int_active == 0)) {
        interruptible_sleep_on_timeout(&sysevent_waitq, timeout * HZ);
    }

    if (fpga_int_active == 1) {     
        interrupt_code = FPGA_INT_CODE;
        fpga_int_active = 0;
        delay = 0;
        goto sysevent_read_return;
    }

    if (pic_int_active == 1) {
        interrupt_code = PIC_INT_CODE;
        pic_int_active = 0;
        delay = 0;
        goto sysevent_read_return;
    }

    /* return with interrupt code == 0 */
    interrupt_code = 0;
    delay = 0;


sysevent_read_return:

    /* delay before returning */
    interruptible_sleep_on_timeout(&delay_waitq, delay * HZ);
    copy_to_user(buf, &interrupt_code, sizeof(int));
    return sizeof(int);

}


/* Write routine
 * -------------
 * Users want to clear a bit mask in the interrupt code. The value should have the bit zero, all other bits are 1
 */
static ssize_t sysevent_write(struct file *file, const char *buf, size_t count, loff_t *offset)
{
    int value;

    copy_from_user(&value, buf, sizeof(int));

    if ((value & I2C_DEBUG_CODE) == I2C_DEBUG_CODE) {
        global_i2c_debug = value & 0xff;
    }

    if ((value & I2C_CLOCK_CODE) == I2C_CLOCK_CODE) {
        i2c_controller_fast_mode = value & 0xff;
    }

    return sizeof(int);
}

/* Ioctl routine */
static int sysevent_ioctl (struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
    return 0;
}


static unsigned int sysevent_poll(struct file *file, poll_table *wait)
{
    return 0;
}



/* seti2croute
 * -----------
 * In 120, i2c bus 1 has a switch that allows access to Summit (channel 0), PS A (channel 1), PS B (channel 2), and Anafa Seeproms (channel 3).
 * The mux select parameter of this function has the value of the channel number, which selects one of these channels.
 */   
int seti2croute(void *vaddr, unsigned char muxselect, int (*iic_xfer)(void *, unsigned short, int, char *, int, int), int controller)
{
    unsigned char data;

    if (muxselect > 3) {
        return 0;
    }

    if (controller == 1) {      /* on 120, controller 1 is NOT connected to the i2c switch */
        return 0;
    }

    data = 1 << muxselect;

    if (iic_xfer(vaddr, (0xff << 8) | LOCAL_SWITCH_I2C_ADDR, WRITEACCESS, &data, 1, controller) != 1) {
        /* return IIC_ERR_LOCALSWITCH_ERR; */
        return 0;	/* to deal with old boards without the switch */ 
    }

    return 0;

}







