/*
 * Linux/PowerPC Real Time Clock Driver
 *
 * heavily based on:
 * Linux/SPARC Real Time Clock Driver
 * Copyright (C) 1996 Thomas K. Dyas (tdyas@eden.rutgers.edu)
 *
 * This is a little driver that lets a user-level program access
 * the PPC clocks chip. It is no use unless you
 * use the modified clock utility.
 *
 * Get the modified clock utility from:
 *   ftp://vger.rutgers.edu/pub/linux/Sparc/userland/clock.c
 */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/miscdevice.h>
#include <linux/slab.h>
#include <linux/fcntl.h>
#include <linux/poll.h>
#include <linux/init.h>
#include <linux/mc146818rtc.h>
#include <asm/system.h>
#include <asm/uaccess.h>
#include <asm/machdep.h>

#include <asm/time.h>

extern int ds1672_set_rtc_time_callback(void *arg);



/* Retrieve the current date and time from the real time clock. */
static void get_rtc_time(struct rtc_time *t)
{
	unsigned long nowtime;

	nowtime = (ppc_md.get_rtc_time)();

	to_tm(nowtime, t);

#if 0
	t->tm_year -= 1900;
	t->tm_mon -= 1; /* Make sure userland has a 0-based month */
#endif
}

/* Set the current date and time in the real time clock. */
static void set_rtc_time(struct rtc_time *t)
{
	unsigned long nowtime;

	nowtime = mktime(t->tm_year+1900, t->tm_mon+1, t->tm_mday,
			t->tm_hour, t->tm_min, t->tm_sec);

#if 0
    nowtime = mktime(t->tm_year, t->tm_mon, t->tm_mday,
            t->tm_hour, t->tm_min, t->tm_sec);
#endif


	/* (ppc_md.set_rtc_time)(nowtime); */
    ds1672_set_rtc_time_callback((void *)nowtime);
}

static int rtc_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
	unsigned long arg)
{
	struct rtc_time rtc_tm;

	switch (cmd)
	{
	case RTC_RD_TIME:
		if (ppc_md.get_rtc_time)
		{
			get_rtc_time(&rtc_tm);

			if (copy_to_user((struct rtc_time*)arg, &rtc_tm, sizeof(struct rtc_time)))
				return -EFAULT;

			return 0;
		}
		else
			return -EINVAL;

	case RTC_SET_TIME:
		if (ppc_md.set_rtc_time)
		{
			if (copy_from_user(&rtc_tm, (struct rtc_time*)arg, sizeof(struct rtc_time)))
				return -EFAULT;

			set_rtc_time(&rtc_tm);

			return 0;
		}
		else
			return -EINVAL;

	default:
		return -EINVAL;
	}
}

static struct file_operations rtc_fops = {
	owner:		THIS_MODULE,
	ioctl:		rtc_ioctl,
};

static struct miscdevice rtc_dev = { RTC_MINOR, "rtc", &rtc_fops };

EXPORT_NO_SYMBOLS;

/* int __init ts_igr_rtc_init(void) */
int ts_igr_rtc_init(void)
{
#if 0
    /* register char device */
    if (register_chrdev(RTC_MAJOR, "rtc module", &rtc_fops)) {
        printk("Unable to register rtc module\n");
        return -EIO;
    }
#endif

    int error;
    static int count = 0;

    count ++;

    if (count > 1) {
        return 0;
    }

    printk(KERN_DEBUG "Register TS_IGR RTC\n");

    error = misc_register(&rtc_dev);
    if (error) {
        printk(KERN_ERR "rtc: unable to get misc minor\n");
        return error;
    }

   
	return 0;
}

void __exit ts_igr_rtc_exit(void)
{
#if 0
    unregister_chrdev(RTC_MAJOR, "rtc module");
#endif
    misc_deregister(&rtc_dev);
}

module_init(ts_igr_rtc_init);
module_exit(ts_igr_rtc_exit);
MODULE_LICENSE("GPL");
