/*
   -------------------------------------------------------------------------
   i2c-algo-ibm_ocp.c i2c driver algorithms for IBM PPC 405 adapters	    
   -------------------------------------------------------------------------
      
   current Mainainers
   Ian DaSilva and Armin Kuster, MontaVista Software, Inc.
   idasilva@mvista.com, Akuster@mvista.com or source@mvista.com

   Copyright 2000-2002 MontaVista Software Inc.

   Changes made to support the IIC peripheral on the IBM PPC 405

   ---------------------------------------------------------------------------
   This file was highly leveraged from i2c-algo-pcf.c, which was created
   by Simon G. Vogl and Hans Berglund:

     Copyright (C) 1995-1997 Simon G. Vogl
                   1998-2000 Hans Berglund

   With some changes from Kysti Mlkki <kmalkki@cc.hut.fi> and 
   Frodo Looijaard <frodol@dds.nl> ,and also from Martin Bailey
   <mbailey@littlefeet-inc.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ---------------------------------------------------------------------------

   History: 01/20/12 - Armin
   	akuster@mvista.com
   	ported up to 2.4.16+	

   Version 02/03/25 - Armin
       converted to ocp format
       removed commented out or #if 0 code
       added Grard Basler's fix to iic_combined_transaction() such that it 
       returns the number of successfully completed transfers .

   Version 04/29/03 - Armin
   	General cleanups
	changes c++ style comments to c
	converted to ocp
	deleted unused printk's
	Added scan routine from Andrew May

   Version 05/25/02 - Armin
      name change for *_driver to *_dev
*/

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <linux/ioport.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <asm/io.h>

#include <linux/i2c.h>
#include <asm/ocp.h>

#include <asm/ibm4xx.h>
#include <asm/bootinfo.h>
#include <linux/i2c-errcode.h>


#define IIC_VER	"2002.30.04"

/* ----- global defines ----------------------------------------------- */
#define DEB(x) if (global_i2c_debug>=1) x
#define DEB2(x) if (global_i2c_debug>=2) x
#define DEB3(x) if (global_i2c_debug>=3) x	/* print several statistical values */
#define DEBPROTO(x) if (global_i2c_debug>=9) x;
	/* debug the protocol by showing transferred bits */
#define DEF_TIMEOUT 0x1000

#define IIC_DEV(x)	((struct ocp_dev *)adap->data)->x

/* ----- global variables ---------------------------------------------	*/
void *fpga_base_addr;
extern int global_i2c_debug;
extern unsigned char i2c_controller_slave_addr;
extern int i2c_controller_fast_mode;
extern wait_queue_head_t iic_ibmocp_bus_waitq;


#ifdef SLO_IO
int jif;
#endif

/* module parameters:
 */
static int i2c_debug = 0;
static int iic_scan = 0;	/* have a look at what's hanging 'round         */

/* --- setting states on the bus with the right timing: ---------------	*/

#define iic_outb( reg, val) writeb(val,&(reg))
#define iic_inb( reg) readb(&(reg))

#define READACCESS              1
#define WRITEACCESS             2

#define IICO_I2C_SDAHIGH	0x0780
#define IICO_I2C_SDALOW		0x0781
#define IICO_I2C_SCLHIGH	0x0782
#define IICO_I2C_SCLLOW		0x0783
#define IICO_I2C_LINEREAD	0x0784

#define IIC_SINGLE_XFER		0
#define IIC_COMBINED_XFER	1


#define MAX_I2C_WAIT		0xffff
#define MAX_I2C_ERROR_RETRY	30
#define I2C_MAX_SCL_TOGGLE      9
#define I2C_MAX_START_STOP_SEQUENCE	3

#define IIC_TIMEOUT		100
#define IIC_RETRY		3

#if defined(CONFIG_TS_270_SC) || defined(CONFIG_TS_120_SC) || defined(CONFIG_TS_BLD_SC)
extern int seti2croute(void *vaddr, unsigned char muxselect, int (*iic_xfer)(void *, unsigned short, int, char *, int, int), int controller);
#endif 

#if defined(CONFIG_TS_BLD_SC)
extern int is_bld_i2cbus1_connected(void);
#endif


extern void simple_iic_ibmocp_waitforpin(int dev_num);

int phy_iic_xfer(void *vaddr, unsigned short dev_addr, int access, char *buf, int count, int controller);

/* --- other auxiliary functions --------------------------------------	*/

/*
 *Description: Puts this process to sleep for a period equal to timeout 
 */
static inline void
iic_sleep(unsigned long timeout)
{
	schedule_timeout(timeout * HZ);
}

int
phy_iic_init(void *vaddr)
{
    struct iic_regs *iic;
    unsigned short retval;
    int data;

    iic = (struct iic_regs *) vaddr;

    /* make sure sda and scl are high */
    iic_outb(iic->directcntl, 0x0c);


    iic_outb(iic->xtcntlss, 0x01);
    udelay(500);

    iic_outb(iic->xtcntlss, 0x00);
    udelay(500);

    /* Clear master low master address */
    iic_outb(iic->lmadr, 0);

    /* Clear high master address */
    iic_outb(iic->hmadr, 0);


    /* set low slave address */
    iic_outb(iic->lsadr, i2c_controller_slave_addr << 1);


    /* Clear high slave address */
    iic_outb(iic->hsadr, 0);

    /* Clear status */
    iic_outb(iic->sts, 0x0a);

    /* Clear extended status */
    iic_outb(iic->extsts, 0x8f);

    /* Set clock division */
    if (i2c_controller_fast_mode != 0) {
        iic_outb(iic->clkdiv, 0x05);
    }
    else {
        iic_outb(iic->clkdiv, 0x04);
    }


    retval = iic_inb(iic->clkdiv);
    DEB(printk("iic_init: CLKDIV register = %x\n", retval));


    if (i2c_controller_slave_addr != 0) {		/* slave mode */
        /* Enable interrupts on Requested Master Transfer Complete, write complete, write needs service, read complete, and read needs service  */
        iic_outb(iic->intmsk, 0xf1);
    }
    else {	/* master only */
        /* Enable interrupts on Requested Master Transfer Complete */
        iic_outb(iic->intmsk, 0x01);
    }


    /* Clear transfer count */
    iic_outb(iic->xfrcnt, 0x0);

    /* Clear extended control and status */
    iic_outb(iic->xtcntlss, 0xf0);


    /* Set mode control */
    data = 0xc6;
    if (i2c_controller_slave_addr != 0) {       /* slave mode */
        data |= 0x09;		/* enable slave mode and hold SCL */
    }

    if (i2c_controller_fast_mode != 0) {
        data |= 0x10;		/* enable fast mode */
    }

    writeb(data, (int) &(iic->mdcntl));    


    /* Clear control register */
    iic_outb(iic->cntl, 0x0);

    DEB2(printk(KERN_DEBUG "iic_init: Initialized IIC on PPC 405\n"));
    return 0;
}


/*
 * Description: This performs the IBM PPC 405 IIC initialization sequence
 * as described in the PPC405GP data book.
 */
static int
iic_init(struct i2c_adapter *adap)
{
	void *vaddr;

	vaddr = (void *) ((struct ocp_dev *) adap->data)->vaddr;

	return phy_iic_init(vaddr);
}

/*
 * Description: After we issue a transaction on the IIC bus, this function
 * is called.  It puts this process to sleep until we get an interrupt from
 * from the controller telling us that the transaction we requested in complete.
 */
static int
wait_for_pin(void *vaddr, int controller)
{
	struct iic_regs *iic;
    volatile int status;
    int timeout = 6;

	iic = (struct iic_regs *) vaddr;

    while (timeout > 0) {
        status = iic_inb(iic->sts);
        status = status & 0x01;         /* transfer pending bit */
        if (status == 0) {
            break;
        }
        DEB3(printk("In wait_for_pin: status = %x\n", status));
        simple_iic_ibmocp_waitforpin(controller);
        timeout --;
    }


    if (timeout != 0) {
        return 0;
    }

    /* if here, time out */
    return (-1);

}



/* ------------------------------------ */
/*  Utility functions */

/*  Description: Look at the status register to see if there was an error 
 *  in the requested transaction.  If there is, look at the extended status 
 *  register and determine the exact cause. 
 */

int
analyze_status(void *vaddr,  int *error_code)
{
	int ret;
	struct iic_regs *iic;

	iic = (struct iic_regs *) vaddr;

    *error_code = IIC_ERR_UNKNOWN;

	ret = iic_inb(iic->sts);

    if ((ret & 0x04) == 0) {
        return 0;
    }

	/* Error occurred */
	ret = iic_inb(iic->extsts);

	if (ret & 0x04) {
		/* Lost arbitration */
		*error_code = IIC_ERR_LOST_ARB;
        return -1;
	}

    if (ret & 0x01) {
        /* Master transfer aborted by a NACK during the transfer of the address byte */
        *error_code = IIC_ERR_NACK;
        return -1;
    }

	if (ret & 0x02) {
		/* Incomplete transfer */
		*error_code = IIC_ERR_INCOMPLETE_XFR;
        return -1;
	}

	return -1;
}

/*  Description: This function is called by the upper layers to do the 
 *  grunt work for a master send transaction
 */

static int
iic_sendbytes(void *vaddr, const char *buf,
	      int count, int xfer_flag, int controller)
{
	struct iic_regs *iic;
	int wrcount, timeout;
	int loops, remainder, i, j;
	int ret, error_code;

	iic = (struct iic_regs *) vaddr;

	if (count == 0)
		return 0;
	wrcount = 0;
	loops = count / 4;
	remainder = count % 4;

	if ((loops > 1) && (remainder == 0)) {
		for (i = 0; i < (loops - 1); i++) {

			/* Write four bytes to master data buffer */

			for (j = 0; j < 4; j++) {
				iic_outb(iic->mdbuf, buf[wrcount++]);
			}

			/* Issue command to IICO device to begin transmission */

			iic_outb(iic->cntl, 0x35);

			/*    Wait for transmission to complete.  When it does,  
			 *    loop to the top of the for statement and write the  
			 *    next four bytes. 
			 */

			timeout = wait_for_pin(vaddr, controller);
			if (timeout < 0) {
				/* Error handling */
				return IIC_ERR_CONTROLLER_CMD_TIMEOUT;
			}
			ret = analyze_status(vaddr, &error_code);
			if (ret < 0) {
			    return error_code;
			}
		}
	} else if ((loops >= 1) && (remainder > 0)) {
		/*printk(KERN_DEBUG "iic_sendbytes: (loops >= 1)\n"); */
		for (i = 0; i < loops; i++) {
			/*
			 * Write four bytes to master data buffer
			 */
			for (j = 0; j < 4; j++) {
				iic_outb(iic->mdbuf, buf[wrcount++]);
			}
			/*
			 * Issue command to IICO device to begin transmission
			 */
			iic_outb(iic->cntl, 0x35);
			/*
			 * Wait for transmission to complete.  When it does,
			 * loop to the top of the for statement and write the
			 * next four bytes.
			 */
			timeout = wait_for_pin(vaddr, controller);
			if (timeout < 0)
				return IIC_ERR_CONTROLLER_CMD_TIMEOUT;

			ret = analyze_status(vaddr, &error_code);
			if (ret < 0) {
				return error_code;
			}
		}
	}

	if (remainder == 0)
		remainder = 4;
	/* Write the remaining bytes (less than or equal to 4) */

	for (i = 0; i < remainder; i++)
		iic_outb(iic->mdbuf, buf[wrcount++]);

	if (xfer_flag == IIC_COMBINED_XFER) {
		iic_outb(iic->cntl, (0x09 | ((remainder - 1) << 4)));
	} else {
		iic_outb(iic->cntl, (0x01 | ((remainder - 1) << 4)));
	}
	DEB2(printk(KERN_DEBUG "iic_sendbytes: Waiting for interrupt\n"));
	timeout = wait_for_pin(vaddr, controller);
	if (timeout < 0)
		return IIC_ERR_CONTROLLER_CMD_TIMEOUT;
	ret = analyze_status(vaddr, &error_code);
	if (ret < 0) {
		return error_code;
	}
	DEB2(printk(KERN_DEBUG "iic_sendbytes: Got interrupt\n"));
	return wrcount;
}

/*
 * Description: Called by the upper layers to do the grunt work for
 * a master read transaction.
 */
static int
iic_readbytes(void *vaddr, char *buf, int count, int xfer_type, int controller)
{
	struct iic_regs *iic;
	int rdcount = 0, i, timeout;
	int loops, remainder, j;
	int ret, error_code;

	iic = (struct iic_regs *) vaddr;

	if (count == 0)
		return 0;
	loops = count / 4;
	remainder = count % 4;

	if ((loops > 1) && (remainder == 0)) {
		for (i = 0; i < (loops - 1); i++) {
			/* Issue command to begin master read (4 bytes maximum) */
			iic_outb(iic->cntl, 0x37);
			/*
			 * Wait for transmission to complete.  When it does,
			 * loop to the top of the for statement and write the
			 * next four bytes.
			 */
			timeout = wait_for_pin(vaddr, controller);
			if (timeout < 0)
				return IIC_ERR_CONTROLLER_CMD_TIMEOUT;

			ret = analyze_status(vaddr, &error_code);
			if (ret < 0) {
				return error_code;
			}

			for (j = 0; j < 4; j++) {
				/* Wait for data to shuffle to top of data buffer
				 * This value needs to optimized.
				 */
				udelay(1);
				buf[rdcount] = iic_inb(iic->mdbuf);
				rdcount++;
			}
		}
	}

	else if ((loops >= 1) && (remainder > 0)) {
		for (i = 0; i < loops; i++) {
			/* Issue command to begin master read (4 bytes maximum) */
			iic_outb(iic->cntl, 0x37);
			/*
			 * Wait for transmission to complete.  When it does,
			 * loop to the top of the for statement and write the
			 * next four bytes.
			 */
			timeout = wait_for_pin(vaddr, controller);
			if (timeout < 0)
				return IIC_ERR_CONTROLLER_CMD_TIMEOUT;

			ret = analyze_status(vaddr, &error_code);
			if (ret < 0) {
				return error_code;
			}

			for (j = 0; j < 4; j++) {
				/* Wait for data to shuffle to top of data buffer
				 * This value needs to optimized.
				 */
				udelay(1);
				buf[rdcount] = iic_inb(iic->mdbuf);
				rdcount++;
			}
		}
	}

	if (remainder == 0)
		remainder = 4;
	DEB2(printk
	     (KERN_DEBUG "iic_readbytes: writing %x to IICO_CNTL\n",
	      (0x03 | ((remainder - 1) << 4))));

	if (xfer_type == IIC_COMBINED_XFER) {
		iic_outb(iic->cntl, (0x0b | ((remainder - 1) << 4)));
	} else {
		iic_outb(iic->cntl, (0x03 | ((remainder - 1) << 4)));
	}
	DEB2(printk(KERN_DEBUG "iic_readbytes: Wait for pin\n"));
	timeout = wait_for_pin(vaddr, controller);
	DEB2(printk(KERN_DEBUG "iic_readbytes: Got the interrupt\n"));
	if (timeout < 0)
		return IIC_ERR_CONTROLLER_CMD_TIMEOUT;

	ret = analyze_status(vaddr, &error_code);
	if (ret < 0) {
		return error_code;
	}

	for (i = 0; i < remainder; i++) {
		buf[rdcount] = iic_inb(iic->mdbuf);
		rdcount++;
	}

	return rdcount;
}




int iic_set_mux(void *vaddr, unsigned short dev_addr, int controller)
{
#ifdef CONFIG_TS_IGR_CC

    unsigned char muxselect;
    unsigned char data;

    muxselect = (dev_addr >> 8) & 0xff;
    if (muxselect != 0xff) {
        /* set the mux on fpga */
        data = readb((void *) ((int) fpga_base_addr + 4));
        data &= 0xf0;   /* clear the mux select part */
        data |= muxselect;
        writeb(data, (void *) ((int) fpga_base_addr + 4));
    }

    return 0;


#endif CONFIG_TS_IGR_CC

#ifdef CONFIG_TS_270_SC

    unsigned char muxselect;
    int ret;

    if (controller == 1) {
        return 0;
    }

    muxselect = (dev_addr >> 8) & 0xff;
    if (muxselect != 0xff) {
        ret = seti2croute(vaddr, muxselect, phy_iic_xfer, controller);
        if (ret != 0) {
            return ret;
        }
    }

  return 0;

#endif CONFIG_TS_270_SC

#ifdef CONFIG_TS_120_SC

    unsigned char muxselect;
    int ret;

    if (controller == 0) {
        return 0;
    }

    muxselect = (dev_addr >> 8) & 0xff;
    if (muxselect != 0xff) {
        ret = seti2croute(vaddr, muxselect, phy_iic_xfer, controller);
        if (ret != 0) {
            return ret;
        }
    }

  return 0;

#endif CONFIG_TS_120_SC

#ifdef CONFIG_TS_BLD_SC

    unsigned char muxselect;
    int ret;

    if (controller == 1) {
        ret = bld_i2cbus1_disconnect();
        if (ret != 0) {	/* fail to disconnect the bus */
            return ret;
        }
        else {
            return 0;
        }
    }

    muxselect = (dev_addr >> 8) & 0xff;
    if (muxselect != 0xff) {
        ret = seti2croute(vaddr, muxselect, phy_iic_xfer, controller);
        if (ret != 0) {
            return ret;
        }
    }

  return 0;

#endif CONFIG_TS_BLD_SC


    return 0;

}






int iic_set_target_address(void *vaddr, unsigned short dev_addr, int access)
{
    struct iic_regs *iic;
    unsigned char addr;

    iic = (struct iic_regs *) vaddr;

    addr = (dev_addr << 1) & 0xff;

    if (access == READACCESS) {
        addr |= 1;
    }

    /*
     * Write to the low slave address
     */
    iic_outb(iic->lmadr, addr);
    /*
     * Write zero to the high slave register since we are
     * only using 7 bit addresses
     */
    iic_outb(iic->hmadr, 0);

    return 0;
}



/*
 * Description:  This function implements combined transactions.  Combined
 * transactions consist of combinations of reading and writing blocks of data.
 * Each transfer (i.e. a read or a write) is separated by a repeated start
 * condition.
 */
static int
iic_combined_transaction(void *vaddr, struct i2c_msg msgs[], int num, int controller)
{
    int i;
    struct i2c_msg *pmsg;
    int ret;
    struct iic_regs *iic;

    iic = (struct iic_regs *) vaddr;

    DEB2(printk(KERN_DEBUG "Beginning combined transaction\n"));
    for (i = 0; i < num; i++) {
        pmsg = &msgs[i];
        if (pmsg->flags & I2C_M_RD) {

            /* Last read or write segment needs to be terminated with a stop */
            if (i < num - 1) {
                DEB2(printk("This one is a read\n"));
            } else {
                DEB2(printk
                     ("Doing the last read\n"));
            }
            ret = iic_set_target_address(vaddr, pmsg->addr, READACCESS);
            if (ret != 0) {
                return ret;
            }
            ret = iic_readbytes(vaddr, pmsg->buf, pmsg->len, (i < (num - 1)) ? IIC_COMBINED_XFER : IIC_SINGLE_XFER, controller);
            if (ret != pmsg->len) {
                DEB2(printk("iic_readbytes failed\n"));
                if (i < num - 1) { 
                    /* if this is not the last transfer in the chain, continue with all transfer left */
                    DEB2(printk("Continue with all transfer left in the chain\n"));
                    continue;
                }
                return ret;
            }
        } else {

            /* Last read or write segment needs to be terminated with a stop */
            if (i < num - 1) {
                DEB2(printk
                     ("This one is a write\n"));
            } else {
                DEB2(printk
                     ("Doing the last write\n"));
            }
            ret = iic_set_target_address(vaddr, pmsg->addr, WRITEACCESS);
            if (ret != 0) {
                return ret;
            }
            ret = iic_sendbytes(vaddr, pmsg->buf, pmsg->len, (i < (num - 1)) ? IIC_COMBINED_XFER : IIC_SINGLE_XFER, controller);
            if (ret != pmsg->len) {
                DEB2(printk("iic_sendbytes failed\n"));
                if (i < num - 1) {
                    /* halt pending transfer (if any) and send stop */
#if 0
                    iic_outb(iic->cntl, 0x80);
                    phy_iic_init(vaddr);
#endif
                    /* if this is not the last transfer in the chain, continue with all transfer left */
                    DEB2(printk("Continue with all transfer left in the chain\n"));
                    continue;

                }
                return ret;
            }
        }
    }

    return num;
}







/* i2c_start_condition
 * -------------------
 * Generate a start condition: switch SDA from high to low while SCL == 1.
 */
int i2c_start_condition(void *vaddr)
{
    struct iic_regs *iic;

    iic = (struct iic_regs *) vaddr;


    /* generate start condition */
    iic_outb(iic->directcntl, 0x0c);	/* sda = 1, scl = 1*/

    udelay(500);

    iic_outb(iic->directcntl, 0x04);    /* sda = 0, scl = 1*/

    udelay(500);

    return 0;

}


/* i2c_stop_condition
 * -------------------
 * Generate a stop condition: switch SDA from low to high while SCL == 1.
 */
int i2c_stop_condition(void *vaddr)
{
    struct iic_regs *iic;

    iic = (struct iic_regs *) vaddr;


    /* generate stop condition */
    iic_outb(iic->directcntl, 0x04);    /* sda = 0, scl = 1*/

    udelay(500);

    iic_outb(iic->directcntl, 0x0c);    /* sda = 1, scl = 1*/

    udelay(500);

    return 0;
}



/* i2c_bus_reset
 * ---------
 * Direct control of the SDA and SCL line. Manually generate the start and stop condition to reset the bus.
 */
int i2c_bus_reset(void *vaddr)
{
    unsigned char sda;
    unsigned char scl;
    int count;
    struct iic_regs *iic;
    unsigned char data;

    iic = (struct iic_regs *) vaddr;


    /* first keep the controller in reset */
    iic_outb(iic->xtcntlss, 0x01);
    udelay(500);

    /* make sure sda and scl are high */
    iic_outb(iic->directcntl, 0x0c);

    /* read sda and scl */
    data = iic_inb(iic->directcntl);

    scl = data & 0x1;       /* bit 7 */
    sda = (data >> 1) & 0x1;       /* bit 6 */

    count = 0;

    while ((sda == 0) || (scl == 0)) {      /* someone holds the line low */
        /* generate start condition */
        i2c_start_condition(vaddr);

        /* generate stop condition */
        i2c_stop_condition(vaddr);

        data = iic_inb(iic->directcntl);
        scl = data & 0x1;       /* bit 7 */
        sda = (data >> 1) & 0x1;       /* bit 6 */

        count ++;

        if (count == I2C_MAX_START_STOP_SEQUENCE) {  /* we have not succeeded gaining control of sda and scl, but go on anyway */
            break;
        }
    }

    if ((sda == 1) && (scl == 1)) {
        /* printk("Bus released after %d start/stop sequence\n", count); */
        /* release the controller out of reset */
        iic_outb(iic->xtcntlss, 0x00);
        udelay(500);
        return 0;
    }

    /* printk("I2C bus hang\n"); */


#if 0

    /* if here, start/stop sequence does not work. Try to toggle SCL */
    count = 0;

    while ((sda == 0) || (scl == 0)) {      /* someone holds the line low */
        /* pull scl low, keep sda high */
        iic_outb(iic->directcntl, 0x08);
        udelay(500);

        /* pull scl high, keep sda high */
        data = 0x0c;
        iic_outb(iic->directcntl, 0x0c); 
        udelay(500);

        data = iic_inb(iic->directcntl);
        scl = data & 0x1;       /* bit 7 */
        sda = (data >> 1) & 0x1;       /* bit 6 */

        count ++;

        if (count == I2C_MAX_SCL_TOGGLE) {  /* we have not succeeded gaining control of sda and scl, but go on anyway */
            break;
        }
    }

    if ((sda == 1) && (scl == 1)) {
        DEB(printk("Bus released after toggling SCL %d times\n", count));
    }
    else {
        DEB(printk("Bus stuck\n"));
    }

    /* generate stop condition */
    i2c_stop_condition(vaddr);

#endif


    /* release the controller out of reset */
    iic_outb(iic->xtcntlss, 0x00);
    udelay(500);

    return 0;

}




/* error_handling
 * --------------
 * Handle error: no ACK, lost arbitration, incomplete transfer, controller timeout when executing a command, bad bus state.
 * The transfer should be retried after error is handled.
 */
int error_handling(void *vaddr, int error_code, int *retry)
{
    struct iic_regs *iic;

    iic = (struct iic_regs *) vaddr;

    /* udelay(200 * (*retry + 1)); */
    interruptible_sleep_on_timeout(&iic_ibmocp_bus_waitq, HZ/100);     /* 10 ms */

    switch (error_code) {
        case IIC_ERR_NACK:
                                     DEB(printk("NACK\n")); 
                                     /* device most likely not present. But retry anyway */
                                     break;
        case IIC_ERR_LOST_ARB:
                                     DEB(printk("Lost ARB\n"));
                                     /* back off then retry */
                                     break;

        case IIC_ERR_CONTROLLER_CMD_TIMEOUT:
                                     DEB(printk("Controller timeout\n"));
                                     *retry = MAX_I2C_ERROR_RETRY;
                                     phy_iic_init(vaddr);

                                     break;

        case IIC_ERR_BUS_BUSY:       
                                     DEB(printk("Bus busy\n"));
                                     /* extra delay when bus is busy */
                                     /* udelay(1000); */
                                     if (*retry >= (MAX_I2C_ERROR_RETRY - 2)) {
                                         phy_iic_init(vaddr);
                                     }
                                     break;

        case IIC_ERR_BUS_BADSTATE:	 DEB(printk("Bus Bad State\n"));
                                     phy_iic_init(vaddr);
                                     break; 

        case IIC_ERR_INCOMPLETE_XFR:	DEB(printk("Incomplete transfer\n"));
                                     if (*retry >= (MAX_I2C_ERROR_RETRY - 2)) {
                                         phy_iic_init(vaddr);
                                     }
                                     break;

        default:					 DEB(printk("Unknown Error\n"));
                                     if (*retry >= (MAX_I2C_ERROR_RETRY - 2)) {
                                         phy_iic_init(vaddr);
                                     }
                                     break;
    }

    return 0;
}



/*
 * Description: Prepares the controller for a transaction (clearing status
 * registers, data buffers, etc), and then calls either iic_readbytes or
 * iic_sendbytes to do the actual transaction.
 */
int
phy_iic_xfer(void *vaddr, unsigned short dev_addr, int access, char *buf, int count, int controller)
{
    struct iic_regs *iic;
    int ret;
    volatile unsigned char status;
    unsigned long timecount;
    int error_retry;
    int max_retry = MAX_I2C_ERROR_RETRY;

    DEB(printk("Single transaction: controller = %x, addr = %x, access = %x\n", controller, dev_addr, access));

    iic = (struct iic_regs *) vaddr;

    if ((ret = iic_set_mux(vaddr, dev_addr, controller)) != 0) {
        return ret;
    }
   
    /* halt pending transfer (if any) and send stop */
    iic_outb(iic->cntl, 0x80);


  for (error_retry = 0; error_retry < max_retry; error_retry ++) {

    /* Clear status register */
    iic_outb(iic->sts, 0x0a);

    /* Wait for any pending transfers to complete */
    timecount = 0;
    while (timecount < MAX_I2C_WAIT) {
        status = iic_inb(iic->sts);
        if ((status & 0x1) == 0) {
            break;
        }
        timecount ++;
    }

    if (timecount == MAX_I2C_WAIT) {
        ret = IIC_ERR_CONTROLLER_CMD_TIMEOUT;
        DEB(printk("Timeout waiting for previous pending transfer\n"));
        error_handling(vaddr, ret, &error_retry);
        continue;		/* next retry */
    }

    /*
     * Flush master data buf
     */
    DEB2(printk("iic_xfer: Clearing master data buffer\n"));
    ret = iic_inb(iic->mdcntl);
    iic_outb(iic->mdcntl, ret | 0x40);

    /*
     * Load slave address
     */
    DEB2(printk("iic_xfer: Loading slave address\n"));
    ret = iic_set_target_address(vaddr, dev_addr, access);
    if (ret != 0) {
        return ret;
    }

    /*
     * Check to see if the bus is busy
     */
    status = iic_inb(iic->extsts);

    status = status & 0x70;
    /* When the bus is free, the BCS bits in the EXT Status register are 0b100 */
    if (status != 0x40) {   /* bus not free */
        if (status == 0x50) {
            ret = IIC_ERR_BUS_BUSY;
            error_handling(vaddr, ret, &error_retry);
            continue;
        }
        else if ((status == 0x0) || (status == 0x70)) {
            ret = IIC_ERR_BUS_BADSTATE;
            error_handling(vaddr, ret, &error_retry);
            continue;
        }
    }


    if (access == READACCESS) {       /* read */
        DEB2(printk(KERN_DEBUG "iic_xfer: Call adapter's read\n"));
        ret = iic_readbytes(vaddr, buf, count, IIC_SINGLE_XFER, controller);
    }
    else {       /* write */
        DEB2(printk(KERN_DEBUG "iic_xfer: Call adapter's write\n"));
        ret = iic_sendbytes(vaddr, buf, count, IIC_SINGLE_XFER, controller);
    }


    if (ret == count) {
        break;      /* no error */
    }


    error_handling(vaddr, ret, &error_retry);

  }     /* end of retry */


    /* udelay(10); */

    return ret;
}

int phy_combined_iic_xfer(void *vaddr, struct i2c_msg msgs[], int num, int controller)
{
    struct iic_regs *iic;
    int ret;
    unsigned char status;
    unsigned long timecount;
    int error_retry;
    int max_retry = MAX_I2C_ERROR_RETRY;
    struct i2c_msg *pmsg;

    pmsg = &msgs[0];

    DEB(printk("Combined transaction: controller = %x, msg0.addr = %x, msg0.flags = %x, msg1.addr = %x, msg1.flags = %x\n", controller, msgs[0].addr, msgs[0].flags, msgs[1].addr, msgs[1].flags));

    iic = (struct iic_regs *) vaddr;

    if ((ret = iic_set_mux(vaddr, pmsg->addr, controller)) != 0) {
        return ret;
    }

    /* halt pending transfer (if any) and send stop */
    iic_outb(iic->cntl, 0x80);


  for (error_retry = 0; error_retry < max_retry; error_retry ++) {

    /*
     * Clear status register
     */
    iic_outb(iic->sts, 0x0a);

    /* Wait for any pending transfers to complete */
    timecount = 0;
    while (timecount < MAX_I2C_WAIT) {
        status = iic_inb(iic->sts);
        if ((status & 0x1) == 0) {
            break;
        }
        timecount ++;
    }

    if (timecount == MAX_I2C_WAIT) {
        ret = IIC_ERR_CONTROLLER_CMD_TIMEOUT;
        DEB(printk("Timeout waiting for previous pending transfer\n"));
        error_handling(vaddr, ret, &error_retry);
        continue;       /* next retry */
    }




    /*
     * Flush master data buf
     */
    DEB2(printk("iic_xfer: Clearing master data buffer\n"));
    ret = iic_inb(iic->mdcntl);
    iic_outb(iic->mdcntl, ret | 0x40);

    /*
     * Check to see if the bus is busy
     */
    status = iic_inb(iic->extsts);

    status = status & 0x70;
    /* When the bus is free, the BCS bits in the EXT Status register are 0b100 */
    if (status != 0x40) {   /* bus not free */
        if (status == 0x50) {
            ret = IIC_ERR_BUS_BUSY;
            error_handling(vaddr, ret, &error_retry);
            continue;
        }
        else if ((status == 0x0) || (status == 0x70)) {
            ret = IIC_ERR_BUS_BADSTATE;
            error_handling(vaddr, ret, &error_retry);
            continue;
        }
    }

    /*
     * Combined transaction (read and write)
     */
    ret = iic_combined_transaction(vaddr, msgs, num, controller);
    if (ret == num) {
        break;
    }

    error_handling(vaddr, ret, &error_retry);


  }     /* end of retry */


    /* udelay(10); */

    return ret;
}




/*
 * Description: Prepares the controller for a transaction (clearing status
 * registers, data buffers, etc), and then calls either iic_readbytes or
 * iic_sendbytes to do the actual transaction.
 */
static int
iic_xfer(struct i2c_adapter *adap, struct i2c_msg msgs[], int num)
{
    void *vaddr;
    int controller;
    struct i2c_msg *pmsg = pmsg = &msgs[0];;

    vaddr = (void *) ((struct ocp_dev *)adap->data)->vaddr;
    controller = (int) ((struct ocp_dev *)adap->data)->num;

    if (num == 1) {
        return phy_iic_xfer(vaddr, pmsg->addr, pmsg->flags, pmsg->buf,  pmsg->len, controller);
    }
    else {
        return phy_combined_iic_xfer(vaddr, msgs, num, controller);
    }

}







/*
 * Description: Implements device specific ioctls.  Higher level ioctls can
 * be found in i2c-core.c and are typical of any i2c controller (specifying
 * slave address, timeouts, etc).  These ioctls take advantage of any hardware
 * features built into the controller for which this algorithm-adapter set
 * was written.  These ioctls allow you to take control of the data and clock
 * lines on the IBM PPC 405 IIC controller and set the either high or low,
 * similar to a GPIO pin.
 */
static int
algo_control(struct i2c_adapter *adap, unsigned int cmd, unsigned long arg)
{
	struct iic_regs *iic;
	int ret = 0;
	int lines;

	iic = (struct iic_regs *) IIC_DEV(vaddr);

	lines = iic_inb(iic->directcntl);

	if (cmd == IICO_I2C_SDAHIGH) {
		lines = lines & 0x01;
		if (lines)
			lines = 0x04;
		else
			lines = 0;
		iic_outb(iic->directcntl, (0x08 | lines));
	} else if (cmd == IICO_I2C_SDALOW) {
		lines = lines & 0x01;
		if (lines)
			lines = 0x04;
		else
			lines = 0;
		iic_outb(iic->directcntl, (0x00 | lines));
	} else if (cmd == IICO_I2C_SCLHIGH) {
		lines = lines & 0x02;
		if (lines)
			lines = 0x08;
		else
			lines = 0;
		iic_outb(iic->directcntl, (0x04 | lines));
	} else if (cmd == IICO_I2C_SCLLOW) {
		lines = lines & 0x02;
		if (lines)
			lines = 0x08;
		else
			lines = 0;
		iic_outb(iic->directcntl, (0x00 | lines));
	} else if (cmd == IICO_I2C_LINEREAD) {
		ret = lines;
	}
	return ret;
}

static u32
iic_func(struct i2c_adapter *adap)
{
	return I2C_FUNC_SMBUS_EMUL | I2C_FUNC_10BIT_ADDR |
	    I2C_FUNC_PROTOCOL_MANGLING;
}

/* -----exported algorithm data: -------------------------------------	*/

static struct i2c_algorithm iic_algo = {
	"IBM on-chip IIC algorithm",
	I2C_ALGO_OCP,
	iic_xfer,
	NULL,
	NULL,			/* slave_xmit           */
	NULL,			/* slave_recv           */
	algo_control,		/* ioctl                */
	iic_func,		/* functionality        */
};

/* 
 * registering functions to load algorithms at runtime 
 */

/*
 * Description: Register bus structure
 */
int
i2c_iic_add_bus(struct i2c_adapter *adap)
{

	DEB2(printk
	     (KERN_DEBUG "i2c-algo-iic.o: hw routines for %s registered.\n",
	      adap->name));
	adap->id |= iic_algo.id;
	adap->algo = &iic_algo;
	adap->timeout = IIC_TIMEOUT;
	adap->retries = IIC_RETRY;

#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif

	iic_init(adap);
	i2c_add_adapter(adap);
	/* scan the I2C bus for valid 7 bit addresses
	 * (ie things that ACK on 1byte read)
	 * if i2c_debug is off we print everything on one line.
	 * if i2c_debug is on we do a newline per print so we don't
	 * clash too much with printk's in the other functions.
	 * TODO: check for 10-bit mode and never run as a slave.
	 */

	if (iic_scan) {

		int found = 0;
		int i;
		printk(KERN_INFO " i2c-algo-iic.o: scanning bus %s. Found ",
		       adap->name);
		if (i2c_debug)
			printk("\n");
		for (i = 0; i < 0x7f; i++) {
			int ret;
			struct i2c_msg msg;
			char data[1];
			msg.addr = i;
			msg.buf = data;
			msg.len = 1;
			msg.flags = I2C_M_RD;
			if ((ret = iic_xfer(adap, &msg, 1)) == 1) {
				if (i2c_debug)
					printk
					    ("I2C Found 0x%02x ret %d Data 0x%02x\n",
					     i, ret, data[0]);
				else
					printk("0x%02x ", i);
				found++;
			}
		}
		if (i2c_debug) {
			if (!found)
				printk("I2C Found Nothing\n");
		} else {
			if (found)
				printk("\n");
			else
				printk("Nothing\n");
		}

	}
	return 1;
}

int
i2c_iic_del_bus(struct i2c_adapter *adap)
{
	int res;
	if ((res = i2c_del_adapter(adap)) < 0)
		return res;
	DEB2(printk
	     (KERN_DEBUG "i2c-algo-iic.o: adapter unregistered: %s\n",
	      adap->name));

#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
	return 1;
}

int __init
i2c_algo_iic_init(void)
{
	printk(KERN_DEBUG "IBM On-chip iic (i2c) algorithm module %s\n",
	       IIC_VER);

	return 0;
}

void
i2c_algo_iic_exit(void)
{
	return;
}

EXPORT_SYMBOL(i2c_iic_add_bus);
EXPORT_SYMBOL(i2c_iic_del_bus);

/*
 * The MODULE_* macros resolve to nothing if MODULES is not defined
 * when this file is compiled.
 */
MODULE_AUTHOR("MontaVista Software <www.mvista.com>");
MODULE_DESCRIPTION("PPC 405 iic algorithm");

MODULE_PARM(iic_scan, "i");
MODULE_PARM(i2c_debug, "i");

MODULE_PARM_DESC(iic_scan, "Scan for active chips on the bus");
MODULE_PARM_DESC(i2c_debug,
		 "debug level - 0 off; 1 normal; 2,3 more verbose; 9 iic-protocol");

MODULE_LICENSE("GPL");

module_init(i2c_algo_iic_init);
module_exit(i2c_algo_iic_exit);
