/***************************************************************************
*
*   Module:		PPCBoot/board/ts_igr_cc
*
*   Filename:           fpga.c
*
*   Author:             Brian Tran
*
*   Created:            09/03/2002
*
*   Description:
*
*   For the FPGA on the Controller card - PPCBoot version
*
****************************************************************************
*                 Source Control System Information
*
*    $Id: fpga.c,v 1.1 2003/10/30 01:38:11 mshannon Exp $
*
****************************************************************************
*                       Modification History
*
*    $Log: fpga.c,v $
*    Revision 1.1  2003/10/30 01:38:11  mshannon
*    +120
*
*
***************************************************************************
*                       (c) COPYRIGHT 2000, 2001, 2002
*                           TopSpin Communications, Inc.
*                           Palo Alto, CA
**************************************************************************/
#ifdef RCSID
static const char rcsid[] = "$Id: fpga.c,v 1.1 2003/10/30 01:38:11 mshannon Exp $";
#endif

#include <ppcboot.h>
#include <osak.h>
#include <malloc.h>
#include "fpga.h"

#define XLNX_IMG_SIGNATURE "XLNX"       // Xilinx sub-partition signature
#define XLNX_SIGNATURE_NAME_LEN         4
#define XLNX_SIGN_OFFSET                8

/* local define */
#define FPGA_PROGRAM_SIGNAL		8
#define FPGA_DONE_SIGNAL		14
#define FPGA_INIT_SIGNAL		7
#define FPGA_DESIGN_FILE_SIZE	157512

#define XLNX_IMG_START_BLOCK 0                          // first block of the XLNX image
#define XLNX_IMG_LENGTH  FPGA_DESIGN_FILE_SIZE          // size of the XLNX image in bytes
#define START_ADD_REG_1 XLNX_IMG_START_BLOCK   
#define START_ADD_REG_2 (START_ADD_REG_1 + (XLNX_IMG_LENGTH >> bdkVol->erasableBlockBits) )
#define FL_BUF_SIZE 512
#define MIN(a,b) 	(((a)<(b)) ? (a) : (b))

/* define return values after executing the commands */
#define COMMAND_SUCCESS		0
#define COMMAND_FAILURE		1

/* define max wait time during fpga programming */
#define MAX_FPGA_PROGRAM_WAIT	0xffff

#define FPGA_DELAY       0xfff

int gpio_set(int bitnum, unsigned int data, int open_drain)
{
    unsigned int read;
    unsigned int bitmask;
    int i;

    /* create the bit mask */
    bitmask = 0x80000000;
    for ( i = 0; i < bitnum; i ++) {
        bitmask = bitmask >> 1;
    }


    if (open_drain == 0) {
	read = in32(GPIO_TCR);
	
        /* make sure to turn off tri-state */
        read = read | bitmask;

        out32(GPIO_TCR, read);
    }
    else {		/* turn on open drain */
	read = in32(GPIO_ODR);

        /* make sure to turn on open-drain */
        read = read | bitmask;

        out32(GPIO_ODR, read);
    }


    /* read output first */
    read = in32(GPIO_OR);

    /* set the desired value, keep other bits unchanged */
    if (data == 0) {
        read = read & (~bitmask);
    }
    else {
        read = read | bitmask;
    }
    
    out32(GPIO_OR, read);

    return COMMAND_SUCCESS;
}


int gpio_get(int bitnum, int *data)
{
    int i;
    unsigned int bitmask;
    unsigned int read;


    /* create the bit mask */
    bitmask = 0x80000000;
    for ( i = 0; i < bitnum; i ++) {
        bitmask = bitmask >> 1;
    }


    read = in32(GPIO_IR);

    if (read & bitmask) {
        *data = 1;
    }
    else {
        *data = 0;
    }

    return COMMAND_SUCCESS;
}

/* toint
 * -----
 */
int toint(char c) 
{
    if (c >= '0' && c <= '9') return c - '0';
    if (c >= 'A' && c <= 'F') return c - 'A' + 10;
    if (c >= 'a' && c <= 'f') return c - 'a' + 10;

    /* c is not a hex digit */
    return 0;

}

int fpga_program(void)
{
    unsigned char config_byte;
    int readback;
    int count;
    unsigned long length = FPGA_DESIGN_FILE_SIZE;
    unsigned long i;
    unsigned long num_read;
    unsigned char *buf;
    volatile void *virt_addr = (volatile void *)TS120SC_FPGA_BASE;
    IOreq ioreq = {0,0,NULL,NULL,0,0};
    BDKStruct bdkStruct;

    /* allocate buffer */
    buf = (unsigned char *)malloc(length);
    if (buf == NULL) {
        printf("FPGA: Ran out of memory\n");
        return -1;
    }
  
    /*
     * Read the Xilinx image
     */
    bdkStruct.startingBlock = START_ADD_REG_1;
    bdkStruct.length        = XLNX_IMG_LENGTH;
    memcpy(bdkStruct.oldSign, XLNX_IMG_SIGNATURE, XLNX_SIGNATURE_NAME_LEN);
    bdkStruct.flags         = EDC;
    bdkStruct.signOffset    = XLNX_SIGN_OFFSET;
    ioreq.irHandle          = CFG_ENV_BDK_PARTITION << 4;
    ioreq.irData            = &bdkStruct;

    if (bdkReadInit(&ioreq) != flOK)
    {
      printf ("FPGA: Failed to read image from DiskOnChip\n");
      goto error_cleanup;
    }

    for (num_read = 0; num_read < XLNX_IMG_LENGTH; num_read += FL_BUF_SIZE)
    {
        bdkStruct.length        = MIN(XLNX_IMG_LENGTH - num_read, FL_BUF_SIZE) ;
	bdkStruct.bdkBuffer     = (byte FAR1 *)(buf + num_read);
        ioreq.irHandle          = CFG_ENV_BDK_PARTITION << 4;
        ioreq.irData            = &bdkStruct;

        if (bdkReadBlock(&ioreq) != flOK)
        {
          printf ("FPGA: Failed reading image from DiskOnChip\n");
          goto error_cleanup;
        }
    }

    /* pull program low. GPIO 8 low, no open drain */
    if (gpio_set(FPGA_PROGRAM_SIGNAL, 0, 0) != COMMAND_SUCCESS) {
            printf ("FPGA: Failed to set PROGRAM_SIGNAL low\n");
            goto error_cleanup;
    }

    /* pull INIT low for delay config */
    if (gpio_set(FPGA_INIT_SIGNAL, 0, 1) != COMMAND_SUCCESS) {
            printf ("FPGA: Failed to set INIT_SIGNAL low\n");
            goto error_cleanup;
    }


    /* check ACK by reading DONE low (GPIO 14) */
    readback = 1;
    count = 0;
    do {
        if (gpio_get(FPGA_DONE_SIGNAL, &readback) != COMMAND_SUCCESS) {
            printf ("FPGA: Failed to read DONE_SIGNAL\n");
            goto error_cleanup;
        }
        count ++;
    } while ((readback == 1) && (count < MAX_FPGA_PROGRAM_WAIT));

    if (readback == 1) {
        printf("FPGA: Time out waiting for DONE low before programming\n");
        goto error_cleanup;
    }


    /* then pull program high. GPIO 8 high, no open drain */
    if (gpio_set(FPGA_PROGRAM_SIGNAL, 1, 0) != COMMAND_SUCCESS) {
        printf ("FPGA: Failed to set INIT_SIGNAL high\n");
        goto error_cleanup;
    }

    /* release INIT */
    if (gpio_set(FPGA_INIT_SIGNAL, 1, 1) != COMMAND_SUCCESS) {
        printf ("FPGA: Failed to release INIT_SIGNAL\n");
        goto error_cleanup;
    }


    /* wait until INIT is high. GPIO 7 is high */
    readback = 0;
    count = 0;
    do {
        if (gpio_get(FPGA_INIT_SIGNAL, &readback) != COMMAND_SUCCESS) {
            printf ("FPGA: Failed to read INIT_SIGNAL\n");
            goto error_cleanup;
        }
//        if (readback == 0) {
//            printf("INIT is still low\n");
//        }
        count ++;
    } while ((readback == 0) && (count < MAX_FPGA_PROGRAM_WAIT));

    if (readback == 0) {
        printf("FPGA: Time out waiting for INIT high before programming\n");
        goto error_cleanup;
    }


    /* start sending data */
    for (i = 0; i < length; i ++) {
        config_byte = toint(buf[i]);	/* first nibble */
        config_byte <<= 4;
        config_byte |= toint(buf[++i]);	/* second nibble */

        *((unsigned char *)virt_addr) = config_byte;

    }

    /* check INIT for CRC Error indication */
    if (gpio_get(FPGA_INIT_SIGNAL, &readback) != COMMAND_SUCCESS) {
        printf ("FPGA: Failed to read INIT_SIGNAL\n");
        goto error_cleanup;
    }

    if (readback == 0) {
        printf("FPGA: INIT low indicating CRC Error after programming\n");
        goto error_cleanup;
    }


    /* read the DONE bit. GPIO 14 - expect high */
    readback = 0;
    count = 0;
    do {
        if (gpio_get(FPGA_DONE_SIGNAL, &readback) != COMMAND_SUCCESS) {
            printf ("FPGA: Failed to read DONE_SIGNAL\n");
            goto error_cleanup;
        }
        count ++;
    } while ((readback == 0) && (count < MAX_FPGA_PROGRAM_WAIT));

    if (readback == 0) {
        printf("FPGA: Time out waiting for DONE high after programming\n");
        goto error_cleanup;
    }

    free(buf);
    return 0;

error_cleanup:
    free(buf);
    printf("POST: FPGA FAILED\n");
    return -1;
}



