/*
*  Copyright (C) 2002 Scott McNutt <smcnutt@artesyncp.com>
*
* See file CREDITS for list of people who contributed to this
* project.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston,
* MA 02111-1307 USA
*/


#include <ppcboot.h>
#include "ts_270_sc.h"
#include <asm/processor.h>
#include "fpga.h"


long int fixed_sdram( void );
void priority_setup(void);

extern int dontboot;
extern int gpio_set(int, unsigned int, int);

/* local defines */
#define ANAFA_1_RESET_GPIO	9
#define ANAFA_2_RESET_GPIO	10
#define CPU_MASTER_REQ		2

int board_pre_init (void)
{
    uint reg;
    
    /*--------------------------------------------------------------------
     * GPIO map
     *
     * GPIO0 PCI ANA1 INTR
     * GPIO1 PCI ANA2 INTR
     * GPIO2 CPU MASTER REQ OUTPUT
     * GPIO3 ENET PHY INTR
     * GPIO4 NC
     * GPIO5 FPGA INTR
     * GPIO6 TEMP VOLTAGE INTR
     * GPIO7 FPGA INIT OUTPUT
     * GPIO8 FPGA PROGRAM ENABLE OUTPUT
     * GPIO9 ANAFA1 RESET OUTPUT
     * GPIO10 ANAFA2 RESET OUTPUT
     * GPIO11 CPU WATCHDOG TOGGLE OUTPUT
     * GPIO12 NC
     * GPIO13 NC
     * GPIO14 FPGA DONE INPUT
     * GPIO15 NC
     * GPIO16 I2C1SL (IBML CLK)
     * GPIO17 I2CSDA (IBML DATA)
     *-------------------------------------------------------------------*/

    /* Configure GPIO0-17 as listed above. Configure GPIO18-31 as gpios */
    mtdcr( cpc0_gpio, 0x21f00000 );

    /*--------------------------------------------------------------------
     * Setup the external bus controller/chip selects
     *-------------------------------------------------------------------*/
    mtdcr( ebccfga, xbcfg );
    reg = mfdcr( ebccfgd );
    reg |= 0x04000000;          /* Set ATC */
    reg |= 0x38000000;          /* Set ready timeout to 2K cycles */
    mtdcr( ebccfgd, reg );

    mtebm( ebmctl, 0x84480000 ); /* External Bus Master; bus width 16bits, parity off */

    mtebc( pb0ap, 0x03814480 );	/* DiskOnChip WBN=1 increase WE setup time */
    mtebc( pb0cr, 0xfff18000 ); /* BA=0xfff 128MB R/W 8bit */
    
    /* Legacy from openbios */
    mtebc( pb1ap, 0x02815480 );	/* NVRAM/RTC */
    mtebc( pb1cr, 0x48018000 ); /* BA=0x480 1MB R/W 8-bit */
    
    mtebc( pb2ap, 0x00000200 );	/* Xilinx FPGA */
    mtebc( pb2cr, 0xf0118000 ); /* BA=0xf01 1MB R/W 8-bit */
    
    mtebc( pb3ap, 0x00000380 );	/* Anafa2 #1 TH=1 RE=1 SOR=1 */
    mtebc( pb3cr, 0xf023a000 ); /* BA=0xf02 2MB R/W 16-bit */

    mtebc( pb4ap, 0x00000380 );	/* Anafa2 #2 TH=1 RE=1 SOR=1 */
    mtebc( pb4cr, 0xf043a000 ); /* BA=0xf02 2MB R/W 16-bit */

    /* Legacy from openbios */
    mtebc( pb7ap, 0x01015280 );	/* FPGA registers */
    mtebc( pb7cr, 0x48318000 ); /* BA=0x483 1MB R/W 8-bit */

    /*--------------------------------------------------------------------
     * Setup the interrupt controller polarities, triggers, etc.
     *-------------------------------------------------------------------*/
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    mtdcr( uic0er, 0x00000000 );    /* disable all */
    mtdcr( uic0cr, 0x00000009 );    /* SMI & UIC1 crit are critical */
    mtdcr( uic0pr, 0xfffffe13 );    /* per ref-board manual */
    mtdcr( uic0tr, 0x01c00008 );    /* per ref-board manual */
    mtdcr( uic0vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic0sr, 0xffffffff );    /* clear all */
    
    mtdcr( uic1sr, 0xffffffff );    /* clear all */
    mtdcr( uic1er, 0x00000000 );    /* disable all */
    mtdcr( uic1cr, 0x00000000 );    /* all non-critical */
    mtdcr( uic1pr, 0xffffe0ff );    /* per ref-board manual */
    mtdcr( uic1tr, 0x00ffc000 );    /* per ref-board manual */
    mtdcr( uic1vr, 0x00000001 );    /* int31 highest, base=0x000 */
    mtdcr( uic1sr, 0xffffffff );    /* clear all */

    priority_setup();

    return 0;
}



int checkboard (void)
{
    sys_info_t sysinfo;
    get_sys_info(&sysinfo);

    printf("Topspin 270 Fabric Switch Card\n");
    printf("\tVCO: %lu MHz\n", sysinfo.freqVCOMhz/1000000);
    printf("\tCPU: %lu MHz\n", sysinfo.freqProcessor/1000000);
    printf("\tPLB: %lu MHz\n", sysinfo.freqPLB/1000000);
    printf("\tOPB: %lu MHz\n", sysinfo.freqOPB/1000000);
    printf("\tEPB: %lu MHz\n", sysinfo.freqEPB/1000000);
    return (0);
}


long int initdram (int board_type)
{
    long    dram_size = 0;
#if defined(CONFIG_SPD_EEPROM)
    dram_size = spd_sdram();
#else
    dram_size = fixed_sdram();
#endif
  return dram_size;
}


#if defined(CFG_DRAM_TEST)
int testdram(void)
{
    uint    *pstart = (uint *)0x00000000;
    uint    *pend   = (uint *)0x08000000;
    uint    *p;

    for( p = pstart; p < pend; p++ )
        *p = 0xaaaaaaaa;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0xaaaaaaaa )
        {
            printf("SDRAM test fails at: %08x\n", (uint)p );
            return 1;
        }
    }

    for( p = pstart; p < pend; p++ )
        *p = 0x55555555;

    for( p = pstart; p < pend; p++ )
    {
        if( *p != 0x55555555 )
        {
            printf("SDRAM test fails at: %08x\n", (uint)p );
            return 1;
        }
    }
    return 0;
}
#endif

#if !defined(CONFIG_SPD_EEPROM)
/*************************************************************************
 *  fixed sdram init -- doesn't use serial presence detect.
 *
 *  Assumes:    128 MB, non-ECC, non-registered
 *              PLB @ 133 MHz
 *
 ************************************************************************/
long int fixed_sdram( void )
{
    return( 128 * 1024 * 1024 );    /* 128 MB                           */
}
#endif /* !defined(CONFIG_SPD_EEPROM) */

extern int fpga_program(void);
extern int faninit(void);

void
misc_init_r(bd_t *bd)
{
    unsigned long *revaddr;
    unsigned int rev, data, slot; 
    unsigned mgmtA = 0;
    unsigned mgmtB = 0;
    unsigned char master = 0;

    // 270SC TODO... check to find out if we should reset Anafas here

    /* Drive Anafa GPIOs low to put into reset */
    printf("Drive Anafa2 #1 and #2 reset low...\n");
    if (gpio_set(ANAFA_1_RESET_GPIO, 0, 1) != 0)
        printf("Error setting Anafa2 #1 reset GPIO low\n");
    if (gpio_set(ANAFA_2_RESET_GPIO, 0, 1) != 0)
        printf("Error setting Anafa2 #1 reset GPIO low\n");
  
    fpga_program();

	/* After FPGA load, change bus timing TWT=2 OEN=2 CSN=0 SOR=1 TH=1 */
    mtebc( pb2ap, 0x01020280 );	/* Xilinx FPGA */

    /* at least verify the signature of the FPGA firmware. Just make sure it is not zero */ 
    revaddr = (unsigned long *) (TS270SC_FPGA_BASE + FPGA_SIGNATURE_OFFSET);
    rev = in8(revaddr);
    
    if ((rev == 0) || (rev == 0xff)) {
        printf("POST: FPGA bad Rev: 0x%x : FAILED\n", rev);
        dontboot = 1;
        return -1;
    } else 
        printf("POST: FPGA Rev:0x%x : PASSED\n", rev);

	/* 
     * Master Arbitration 
     */

    /* Find out which mgmt i/o modules are present */
    data = in8((unsigned long *) (TS270SC_FPGA_BASE + FPGA_MOD_PRES_REG2_OFFSET));
    mgmtA = (data & 0x0f) >> 3;
    mgmtB = (data & 0x07) >> 2;

    /* Read module and chassis ID register - master status */
    data = in8((unsigned long *) (TS270SC_FPGA_BASE + FPGA_MOD_CHASID_OFFSET));
    slot = data & 0x0f;

    /* 
     * If I'm a core, and the other core is not a master, request mastership.
     */

    /* If slot 1 (B), delay to avoid master contention */
    if (slot == 1) {
        printf ("HW Slot 1. Delay 1 second...\n");
        udelay(3000000);
    }

    /* Read again to see if master status has changed */
    data = in8((unsigned long *) (TS270SC_FPGA_BASE + FPGA_MOD_CHASID_OFFSET));
    slot = data & 0x0f;

    if ((slot == 0) || (slot == 1)) {
        printf("HW Slot %d. I'm a core. ", slot);
        if (data & 0x40) {
            printf("The other core is master.\n");
            /* request master anyway, in case the other core gives it up */
            gpio_set(CPU_MASTER_REQ, 0, 0);
            udelay(1000);
            gpio_set(CPU_MASTER_REQ, 1, 0);
            udelay(1000);
        } else { 
            if (data & 0x80) {
                printf ("I am master.\n");
                /* If the core doesn't have a mgmt I/O relinquish mastership */
                if (((slot == 0) && (!mgmtA)) || ((slot == 1) && (!mgmtB))) {
                    gpio_set(CPU_MASTER_REQ, 0, 0);
                    master = 0;
                } else
                    master = 1;
            } else {
                if (((slot == 0) && (!mgmtA)) || ((slot == 1) && (!mgmtB))) {
                    printf("No corresponding mgmt I/O present, don't request mastership.\n");
                } else {
                    printf("The other core is NOT master. Request mastership.\n");
                    gpio_set(CPU_MASTER_REQ, 0, 0);
                    udelay(1000);
                    gpio_set(CPU_MASTER_REQ, 1, 0);
                    udelay(1000);
                    data = in8((unsigned long *) (TS270SC_FPGA_BASE + FPGA_MOD_CHASID_OFFSET));
                    if (data & 0x80) {
                        printf ("I am master.\n");
                        master = 1;
                    } else {
                        printf ("POST: Request mastership: FAILED.\n");
                    }
                }
            }
        }
        if (master) {
            /* Turn off nodes, until chassis mgr comes up */
            data = in8((unsigned long *) (TS270SC_FPGA_BASE + FPGA_MOD_POWER_OFFSET));
            data = data & 0x0f;
            out8((unsigned long *) (TS270SC_FPGA_BASE + FPGA_MOD_POWER_OFFSET), data);
        }
    } else {
        printf("FPGA Slot %d. I'm a node. ", slot);
    }

}

