/*---------------------------------------------------------------------*/
/*     DiskOnchip 2000 & Millenium - DiskOnChip-Copier                 */
/*---------------------------------------------------------------------*/

#include <ppcboot.h>
#if defined(CONFIG_TSIGRCC) || defined(CONFIG_TSIGRSC) || defined(CONFIG_TSIGREN) || defined(CONFIG_TSIGRFG) || defined(CONFIG_TSIGRFC) || defined(CONFIG_TSIGRLT) || defined(CONFIG_TS270SC) || defined(CONFIG_TS120SC) || defined(CONFIG_TSBLDSC) || defined(CONFIG_TSIGRNW)
#include <osak.h>
#include <malloc.h>

#define BDK_IMG_SIGNATURE "BIPO"        // default signature of BDK partition that contains the registry
#define BDK_SIGNATURE_NAME_LEN         4
#define BDK_SIGN_OFFSET                8
#define PENV_IMG_SIGNATURE "PENV"       // PPC Environment sub-partition signature
#define PENV_SIGNATURE_NAME_LEN         4
#define PENV_SIGN_OFFSET                8

#define REGISTRY_IMG_START_BLOCK (0)   // first block to store registry
#define REGISTRY_IMG_LENGTH CFG_ENV_SIZE //size of registry in bytes
#define START_ADD_REG_1 REGISTRY_IMG_START_BLOCK   
#define START_ADD_REG_2 (START_ADD_REG_1 + (REGISTRY_IMG_LENGTH >> bdkVol->erasableBlockBits) )

/* Early malloc memory area */
extern unsigned char *early_malloc_p;

void
mdocp_env_init(init_data_t *idata)
{
  IOreq ioreq = {0,0,NULL,NULL,0,0};
  BDKStruct bdkStruct;
	ulong crc;
	unsigned char *data;
  
  /*
   * Read the primary registry record
   */
  bdkStruct.startingBlock = START_ADD_REG_1;
  bdkStruct.length        = REGISTRY_IMG_LENGTH;
  memcpy(bdkStruct.oldSign, PENV_IMG_SIGNATURE, PENV_SIGNATURE_NAME_LEN);
  bdkStruct.flags         = EDC;
  bdkStruct.signOffset    = PENV_SIGN_OFFSET;
  ioreq.irHandle          = CFG_ENV_BDK_PARTITION << 4;
  ioreq.irData            = &bdkStruct;

  if (bdkReadInit(&ioreq) != flOK)
  {
    flExit();
    return;
  }

  bdkStruct.startingBlock = START_ADD_REG_1;
  bdkStruct.length        = REGISTRY_IMG_LENGTH;
  bdkStruct.bdkBuffer     = (byte FAR1 *)CFG_INIT_ENV_ADDR;
  ioreq.irHandle          = CFG_ENV_BDK_PARTITION << 4;
  ioreq.irData            = &bdkStruct;

  if (bdkReadBlock(&ioreq) != flOK)
  {
    flExit();
    return;
  }

  crc = *(ulong *)CFG_INIT_ENV_ADDR;
  data = (unsigned char *)(CFG_INIT_ENV_ADDR + sizeof(long));

	if (crc32(0, data, REGISTRY_IMG_LENGTH - sizeof(long)) == crc)
	{
		idata->env_addr  = (ulong)CFG_INIT_ENV_ADDR + sizeof(long);
		idata->env_valid = 1;
	}
	
  flExit();

} /* mdocp_env_init */

int
mdocp_env_write(uchar *ptr, int size)
{
  IOreq ioreq = {0,0,NULL,NULL,0,0};
  BDKStruct bdkStruct;
	
	/* Make sure we use the real malloc */
  early_malloc_p = NULL;

  /*
   * Update the primary registry record
   */
  bdkStruct.startingBlock = START_ADD_REG_1;
  bdkStruct.length        = size;
  memcpy(bdkStruct.oldSign, PENV_IMG_SIGNATURE, PENV_SIGNATURE_NAME_LEN);
  bdkStruct.flags         = BDK_COMPLETE_IMAGE_UPDATE|EDC;
  bdkStruct.signOffset    = PENV_SIGN_OFFSET;
  ioreq.irHandle          = CFG_ENV_BDK_PARTITION << 4;
  ioreq.irData            = &bdkStruct;

  if (bdkWriteInit(&ioreq) != flOK)
    return 1;

  bdkStruct.bdkBuffer     = (byte FAR1 *)ptr;
  bdkStruct.flags         = ERASE_BEFORE_WRITE;
  ioreq.irHandle          = CFG_ENV_BDK_PARTITION << 4;
  ioreq.irData            = &bdkStruct;

  if (bdkWriteBlock(&ioreq) != flOK)
    return 1;

  return 0;

} /* mdocp_env_write */

#endif /* defined(CONFIG_TSIGRSC) || defined(CONFIG_TSIGRCC) ||
          defined(CONFIG_TSIGREN) || defined(CONFIG_TSIGRFC) || 
          defined(CONFIG_TSIGRLT) || defined(CONFIG_TS270SC) ||
	  defined(CONFIG_TS120SC) */

