/* mainLib/priority.c, os_open_spru_iolib, osop_6m_spru 11/5/98 13:13:51 */
/*-----------------------------------------------------------------------------+
|
|       This source code has been made available to you by IBM on an AS-IS
|       basis.  Anyone receiving this source is licensed under IBM
|       copyrights to use it in any way he or she deems fit, including
|       copying it, modifying it, compiling it, and redistributing it either
|       with or without modifications.  No license under IBM patents or
|       patent applications is to be implied by the copyright license.
|
|       Any user of this software should understand that IBM cannot provide
|       technical support for this software and will not be responsible for
|       any consequences resulting from the use of this software.
|
|       Any person who transfers this source code or any derivative work
|       must include the IBM copyright notice, this paragraph, and the
|       preceding two paragraphs in the transferred software.
|
|       COPYRIGHT   I B M   CORPORATION 1992, 1996
|       LICENSED MATERIAL  -  PROGRAM PROPERTY OF I B M
+-----------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------+
|
|  File Name:   priority.c
|
|  Function:    440GP priority settings
|
|  Author:      Dan Goehring 
|
|  Change Activity-
|
|  Date        Description of Change                                       BY
|  ---------   ---------------------                                       ---
|  30-Aug-01   Created                                                     dwg
+-----------------------------------------------------------------------------*/

#include <ppcboot.h>
#include <asm/processor.h>
#include <openbios/a440gp.h>

void priority_setup(void)
{
  unsigned long temp;

#if 1
  /* Set the PLB priority mode to fair */
  temp = mfdcr(PLB0_ACR);
  temp &= ~PLB_ACR_PPM_MASK;
  temp |= PLB_ACR_PPM_FAIR;
  mtdcr(PLB0_ACR, temp);

  /* Set up alternate PLB master priority settings */
  temp = mfdcr(CPC0_PLB);
  temp &= ~(CPC0_PLB_APM0_MASK | CPC0_PLB_APM1_MASK | 
            CPC0_PLB_APM2_MASK | CPC0_PLB_APM3_MASK | 
                                 CPC0_PLB_APM5_MASK | 
            CPC0_PLB_APM6_MASK | CPC0_PLB_APM7_MASK | 
            CPC0_PLB_M0PS_MASK | CPC0_PLB_M1PS_MASK | 
            CPC0_PLB_M2PS_MASK | CPC0_PLB_M3PS_MASK | 
                                 CPC0_PLB_M5PS_MASK | 
            CPC0_PLB_M6PS_MASK | CPC0_PLB_M7PS_MASK | 
            CPC0_PLB_SAP_MASK  | CPC0_PLB_DAP_MASK  | CPC0_PLB_POBAP_MASK);
  temp |=   CPC0_PLB_APM0_0    | CPC0_PLB_APM1_0    | 
            CPC0_PLB_APM2_0    | CPC0_PLB_APM3_0    | 
                                 CPC0_PLB_APM5_0    | 
            CPC0_PLB_APM6_0    | CPC0_PLB_APM7_0    | 
            CPC0_PLB_M0PS_ALT  | CPC0_PLB_M1PS_ALT  | 
            CPC0_PLB_M2PS_ALT  | CPC0_PLB_M3PS_ALT  |
                                 CPC0_PLB_M5PS_ALT  | 
            CPC0_PLB_M6PS_ALT  | CPC0_PLB_M7PS_ALT  | 
            CPC0_PLB_SAP_ENAB  | CPC0_PLB_DAP_ENAB  | CPC0_PLB_POBAP_ENAB;
  mtdcr(CPC0_PLB, temp);

  /* Processor core instruction cache unit. */
  /*   IRF. IcuRdFetch priority setting */
  /*   IRT. IcuRdTouch priority setting */
  /*   IRS. IcuRdSpec priority setting  */

  /* Processor core data cache read unit. */
  /*   DRU. DcuRdUrgent priority setting */
  /*   DRT. DcuRdTouch priority setting  */
  /*  DRNC. DcuRdNonCache priority       */
  /*  DRLC. DcuRdLdCache priority setting*/
  
  /* Processor core data cache write unit*/
  /*   DWF. DcuWrFlush priority setting  */
  /*   DWS. DcuWrStore priority setting  */
  /*   DWU. DcuWrUrgent priority setting */
 
  temp = mfdcr(CPC0_CR1);
  temp &= ~(CPC0_CR1_DRU_MASK  | CPC0_CR1_DRT_MASK  | 
            CPC0_CR1_DRNC_MASK | CPC0_CR1_DRLC_MASK | 
            CPC0_CR1_DWF_MASK  | CPC0_CR1_DWS_MASK  | 
            CPC0_CR1_DWU_MASK  | CPC0_CR1_IRF_MASK  | 
            CPC0_CR1_IRT_MASK  | CPC0_CR1_IRS_MASK);
  temp |= CPC0_CR1_DRU_0  | CPC0_CR1_DRT_0  | 
          CPC0_CR1_DRNC_0 | CPC0_CR1_DRLC_0 | 
          CPC0_CR1_DWF_0  | CPC0_CR1_DWS_0  | 
          CPC0_CR1_DWU_0  | CPC0_CR1_IRF_0  | 
          CPC0_CR1_IRT_0  | CPC0_CR1_IRS_0;
  mtdcr(CPC0_CR1, temp); 

  /* Peripheral component interconnect bridge controller */
// Performed by the Linux
//  temp = mfdcr(PCIX0_BRDGOPT1);
//  temp &= ~PCIX_BRDGOPT1_PLREQ_MASK;
//  temp |= PCIX_BRDGOPT1_PLREQ_0;
//  mtdcr(PCIX0_BRDGOPT1, temp);

  /* Media access layer */
  temp = mfdcr(MAL0_CFG);
  if ((get_pvr() == PVR_440GP_PASS_1_0) || (get_pvr() == PVR_440GP_PASS_1_1))
  {
    temp &= ~MAL_P1_CFG_PLBP_MASK;
    temp |= MAL_P1_CFG_PLBP_0;
  }
  else
  {
    temp &= ~(MAL_P2_CFG_RPP_MASK | MAL_P2_CFG_WPP_MASK);
    temp |= MAL_P2_CFG_RPP_0 | MAL_P2_CFG_WPP_0;
  }
  mtdcr(MAL0_CFG, temp);

  /* Set DMA memory accesses to a specific priority */
  temp = mfdcr(DMA0_CR0);
  temp &= ~DMA_CR_CP_MASK;
  temp |= DMA_CR_CP_0;
  mtdcr(DMA0_CR0, temp);

  temp = mfdcr(DMA0_CR1);
  temp &= ~DMA_CR_CP_MASK;
  temp |= DMA_CR_CP_0;
  mtdcr(DMA0_CR1, temp);

  temp = mfdcr(DMA0_CR2); 
  temp &= ~DMA_CR_CP_MASK;
  temp |= DMA_CR_CP_0;
  mtdcr(DMA0_CR2, temp);

  temp = mfdcr(DMA0_CR3); 
  temp &= ~DMA_CR_CP_MASK;
  temp |= DMA_CR_CP_0;
  mtdcr(DMA0_CR3, temp);

  /* OPB to PLB bridge controller */
  temp = mfdcr(OPB0_CTRL);
  temp &= ~OPB_PRI_MASK;
  temp |= OPB_PRI_0;
  mtdcr(OPB0_CTRL, temp);
#endif
}

