/*
 * (C) Copyright 2002 Scott McNutt <smcnutt@artesyncp.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/************************************************************************
 * board/config_EBONY.h - configuration for IBM 440GP Ref (Ebony)
 ***********************************************************************/

#ifndef __CONFIG_H
#define __CONFIG_H

/*-----------------------------------------------------------------------
 * High Level Configuration Options
 *----------------------------------------------------------------------*/
#define CONFIG_TS270SC          1	          /* Board is Topspin 270 SC  */
#define CONFIG_TS270SC_RAM	    1	          /* Using SDRAM only, no flash devices available	*/
#define CONFIG_4xx              1           /* ... PPC4xx family        */
#define CONFIG_BOARD_PRE_INIT               /* Call board_pre_init      */
#undef  CFG_DRAM_TEST                       /* Disable-takes long time! */
#define CONFIG_SYS_CLK_FREQ     33333333    /* external freq to pll     */

/*-----------------------------------------------------------------------
 * Base addresses -- Note these are effective addresses where the
 * actual resources get mapped (not physical addresses)
 *----------------------------------------------------------------------*/
#define CFG_SDRAM_BASE      0x00000000      /* _must_ be 0              */
#define CFG_FLASH_BASE      0xff800000      /* start of FLASH           */
#ifndef CONFIG_TS270SC_RAM
#define CFG_MONITOR_BASE    0xfff80000      /* start of monitor         */
#else
#define CFG_MONITOR_BASE    0x100000        /* start of monitor         */
#endif
#define CFG_PERIPHERAL_BASE 0xe0000000      /* internal peripherals     */
#define CFG_ISRAM_BASE      0xc0000000      /* internal SRAM            */

#define CFG_FPGA_BASE       (CFG_PERIPHERAL_BASE + 0x08300000)
#define CFG_NVRAM_BASE_ADDR (CFG_PERIPHERAL_BASE + 0x08000000)

/*-----------------------------------------------------------------------
 * Initial RAM & stack pointer (placed in internal SRAM)
 *----------------------------------------------------------------------*/
#define CFG_INIT_RAM_ADDR   CFG_ISRAM_BASE  /* Initial RAM address      */
#define CFG_INIT_RAM_END    0x2000          /* End of used area in RAM  */
#define CFG_INIT_DATA_SIZE  128             /* num bytes initial data   */

#define CFG_INIT_DATA_OFFSET    (CFG_INIT_RAM_END - CFG_INIT_DATA_SIZE)
#define CFG_INIT_SP_OFFSET      CFG_INIT_DATA_OFFSET

//#define CFG_MONITOR_LEN     (192 * 1024)    /* Reserve 192 kB for Mon   */
#define CFG_MALLOC_LEN        (768 * 1024)    /* Reserve 256 kB for malloc*/
#define CFG_MONITOR_LEN		(512 * 1024)	/* Reserve 512 kB for Monitor	*/
//#define CFG_MALLOC_LEN		(768 * 1024)	/* Reserve 768 kB for malloc()*/

/*-----------------------------------------------------------------------
 * Serial Port
 *----------------------------------------------------------------------*/
#undef  CONFIG_SERIAL_SOFTWARE_FIFO
#define CFG_EXT_SERIAL_CLOCK    3646800
#define CONFIG_BAUDRATE         9600

#define CFG_BAUDRATE_TABLE  \
    {300, 600, 1200, 2400, 4800, 9600, 19200, 38400}
#define CFG_CONSOLE_INFO_QUIET

/*-----------------------------------------------------------------------
 * NVRAM/RTC
 *
 * NOTE: Upper 8 bytes of NVRAM is where the RTC registers are located.
 * The DS1743 code assumes this condition (i.e. -- it assumes the base
 * address for the RTC registers is:
 *
 *      CFG_NVRAM_BASE_ADDR + CFG_NVRAM_SIZE
 *
 *----------------------------------------------------------------------*/
#define CFG_NVRAM_SIZE      (0x2000 - 8)    /* NVRAM size(8k)- RTC regs */
#define CONFIG_RTC_DS174x	1	            /* DS1743 RTC               */

/*-----------------------------------------------------------------------
 * FLASH related
 *----------------------------------------------------------------------*/
#define CFG_MAX_FLASH_BANKS     3	            /* number of banks      */
#define CFG_MAX_FLASH_SECT      32	            /* sectors per device   */

#undef  CFG_FLASH_CHECKSUM
#define CFG_FLASH_ERASE_TOUT	120000	/* Timeout for Flash Erase (in ms)	*/
#define CFG_FLASH_WRITE_TOUT	500	/* Timeout for Flash Write (in ms)	*/

/*-----------------------------------------------------------------------
 * Environment
 *----------------------------------------------------------------------*/
#define CFG_ENV_IS_IN_MDOCP  1	            /* Env in  DiskOnChip 	    */
#undef  CFG_ENV_IS_IN_NVRAM                 /* ... not in  NVRAM        */
#undef  CFG_ENV_IS_IN_FLASH                 /* ... not in flash         */
#undef  CFG_ENV_IS_IN_EEPROM                /* ... not in EEPROM        */

#if !defined(CFG_ENV_IS_IN_MDOCP)
#define CFG_ENV_SIZE            0x1000      /* Size of Environment vars	*/
#define CFG_ENV_ADDR		\
	(CFG_NVRAM_BASE_ADDR+CFG_NVRAM_SIZE-CFG_ENV_SIZE)
#endif

#undef  CONFIG_BOOTARGS
#define CONFIG_BOOTCOMMAND                          \
    "tftpboot 0x200000; "                             \
    "setenv bootargs root=/dev/nfs rw nfsroot=$(rootpath) "  \
    "ip=$(ipaddr):$(serverip):$(gatewayip):" \
    "$(netmask):$(hostname):eth0:none; "\
    "bootm"

#define CONFIG_ENV_OVERWRITE	/* allow changes to ethaddr (for now)	*/
    
#if CONFIG_TS270SC_RAM
/* first ethernet */
#define CONFIG_ETHADDR          00:05:ad:00:11:75

/* next two ethernet hwaddrs */
#define CONFIG_ETH1ADDR         00:11:22:33:44:99
#define CONFIG_ETH2ADDR         00:11:22:33:44:AA

#define CONFIG_IPADDR           10.3.0.47
#define CONFIG_SERVERIP         10.3.0.83
#define CONFIG_GATEWAYIP   10.3.0.1
#define CONFIG_NETMASK     255.255.255.0
#define CONFIG_HOSTNAME    igr-cc-ram
#define CONFIG_BOOTFILE    /tftpboot/mclx/vmlinux-440-test
#define CONFIG_ROOTPATH    10.3.0.83:/opt/hardhat/devkit/ppc/405/target
#endif


#define CONFIG_BOOTDELAY	3	      /* autoboot after 3 seconds	*/
#define CONFIG_BAUDRATE		9600

#define CONFIG_LOADS_ECHO	1	/* echo on for serial download	*/
#define CFG_LOADS_BAUD_CHANGE	1	/* allow baudrate change	*/

#define CONFIG_MII		1	/* MII PHY management		*/
#define	CONFIG_PHY_ADDR		0x9	/* PHY address AD4=0, AD3=1, AD2=0, PHYAD_ST=0 */


#define CONFIG_COMMANDS	       (CONFIG_CMD_DFL	| \
				CFG_CMD_IRQ	| \
				CFG_CMD_I2C	| \
				CFG_CMD_SRAM	| \
				CFG_CMD_KGDB	| \
				CFG_CMD_DHCP	| \
				CFG_CMD_DATE	| \
				CFG_CMD_BEDBUG	| \
				CFG_CMD_MBOOT	| \
				CFG_CMD_ANAFA	| \
				CFG_CMD_ELF	)

/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>

#undef CONFIG_WATCHDOG			/* watchdog disabled		*/

#undef CONFIG_SPD_EEPROM       /* don't use SPD EEPROM for setup    */

#if !defined(CONFIG_SPD_EEPROM)
#define SDRAM_MEM_SIZE  (32*1024*1024)
#endif

/*
 * Miscellaneous configurable options
 */
#define CFG_LONGHELP			/* undef to save memory		*/
#define CFG_PROMPT	"=> "		/* Monitor Command Prompt	*/
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define	CFG_CBSIZE	1024		/* Console I/O Buffer Size	*/
#else
#define	CFG_CBSIZE	256		/* Console I/O Buffer Size	*/
#endif
#define CFG_PBSIZE (CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define CFG_MAXARGS	16		/* max number of command args	*/
#define CFG_BARGSIZE	CFG_CBSIZE	/* Boot Argument Buffer Size	*/

#define CFG_MEMTEST_START	0x0400000	/* memtest works on	*/
#define CFG_MEMTEST_END		0x0C00000	/* 4 ... 12 MB in DRAM	*/

#define CFG_LOAD_ADDR		0x100000	/* default load address */
#define CFG_EXTBDINFO		1	/* To use extended board_into (bd_t) */

#define	CFG_HZ		1000		/* decrementer freq: 1 ms ticks	*/

#define CONFIG_HARD_I2C		1	/* I2C with hardware support	*/
#undef  CONFIG_SOFT_I2C			/* I2C bit-banged		*/
#define CFG_I2C_SPEED		400000	/* I2C speed and slave address	*/
#define CFG_I2C_SLAVE		0x7F


/*-----------------------------------------------------------------------
 * PCI stuff
 *-----------------------------------------------------------------------
 */
#if 0
#define PCI_HOST_ADAPTER 0              /* configure ar pci adapter     */
#define PCI_HOST_FORCE  1               /* configure as pci host        */
#define PCI_HOST_AUTO   2               /* detected via arbiter enable  */

#define CONFIG_PCI			/* include pci support	        */
#define CONFIG_PCI_HOST	PCI_HOST_FORCE  /* select pci host function     */
#define CONFIG_PCI_PNP			/* do pci plug-and-play         */
                                        /* resource configuration       */

#define CFG_PCI_SUBSYS_VENDORID 0x0000  /* PCI Vendor ID: to-do!!!      */
#define CFG_PCI_SUBSYS_DEVICEID 0x0000  /* PCI Device ID: to-do!!!      */
#define CFG_PCI_PTM1LA  0x00000000      /* point to sdram               */
#define CFG_PCI_PTM1MS  0x80000001      /* 2GB, enable hard-wired to 1  */
#define CFG_PCI_PTM1PCI 0x00000000      /* Host: use this pci address   */
#define CFG_PCI_PTM2LA  0x00000000      /* disabled                     */
#define CFG_PCI_PTM2MS  0x00000000      /* disabled                     */
#define CFG_PCI_PTM2PCI 0x04000000      /* Host: use this pci address   */
#endif

/* DiskOnChip Millenium Plus contains the environment */
#ifdef CFG_ENV_IS_IN_MDOCP
/* #define CFG_ENV_BDK_PARTITION	 2 */       /* Binary partition number          */
#define CFG_ENV_BDK_PARTITION	 0       /* Binary partition number          */
                                       /* used to store the environment    */
#define	CFG_ENV_SIZE		       32768   /* Total Size of the Environment Sector	*/
#define CFG_INIT_ENV_ADDR      (CFG_MONITOR_BASE - CFG_ENV_SIZE)
                                        /* Initial env buffer in the memory */
#define CFG_INIT_PRIM_ENV_ADDR (CFG_INIT_ENV_ADDR - CFG_ENV_SIZE)
#define CFG_INIT_BACK_ENV_ADDR (CFG_INIT_ENV_ADDR - 2 * CFG_ENV_SIZE)
                                        /* Temp env buffers in the memory */
#define CFG_INIT_MALLOC_ADDR   (CFG_INIT_BACK_ENV_ADDR - CFG_MALLOC_LEN)
#endif /* CFG_ENV_IS_IN_MDOCP */

/*
 * For booting Linux, the board info and command line data
 * have to be in the first 8 MB of memory, since this is
 * the maximum mapped by the Linux kernel during initialization.
 */
#define CFG_BOOTMAPSZ		(8 << 20)	/* Initial Memory map for Linux */
/*-----------------------------------------------------------------------
 * Cache Configuration
 */
#define CFG_DCACHE_SIZE		8192	/* For IBM 405 CPUs			*/
#define CFG_CACHELINE_SIZE	32	/* ...			*/
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CFG_CACHELINE_SHIFT	5	/* log base 2 of the above value	*/
#endif


/* Configuration Port location */
#define CONFIG_PORT_ADDR	0xF0000500

/*-----------------------------------------------------------------------
 * Definitions for Serial Presence Detect EEPROM address
 * (to get SDRAM settings)
 */
#define SPD_EEPROM_ADDRESS      0x50

/*
 * Recovery Image Definitions
 */
#define CONFIG_RECOVERY
#define TS_BOOTSIG_ADDR       ((unsigned int *) 0xc0000100)
#define TS_BOOTSIG_RECOVERY   0x1234abcd
#define TS_ROOTPARTSIG_ADDR   ((unsigned int *) 0xc0000120)
#define TS_ROOTPARTSIG_VALUE  0x12345678

/*
 * Internal Definitions
 *
 * Boot Flags
 */
#define BOOTFLAG_COLD	0x01		/* Normal Power-On: Boot from FLASH	*/
#define BOOTFLAG_WARM	0x02		/* Software reboot			*/

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CONFIG_KGDB_BAUDRATE	230400	/* speed to run kgdb serial port */
#define CONFIG_KGDB_SER_INDEX	2	/* which serial port to use */
#endif
#endif	/* __CONFIG_H */
