/* m4/mal.inc, openbios_ebony, ebony_bios 7/9/00 14:37:48 
!------------------------------------------------------------------------------+
!
!       This source code has been made available to you by IBM on an AS-IS
!       basis.  Anyone receiving this source is licensed under IBM
!       copyrights to use it in any way he or she deems fit, including
!       copying it, modifying it, compiling it, and redistributing it either
!       with or without modifications.  No license under IBM patents or
!       patent applications is to be implied by the copyright license.
!
!       Any user of this software should understand that IBM cannot provide
!       technical support for this software and will not be responsible for
!       any consequences resulting from the use of this software.
!
!       Any person who transfers this source code or any derivative work
!       must include the IBM copyright notice, this paragraph, and the
!       preceding two paragraphs in the transferred software.
!
!       COPYRIGHT   I B M   CORPORATION 1995
!       LICENSED MATERIAL  -  PROGRAM PROPERTY OF I B M
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
!  File Name:   mal.inc 
!
!  Function:    Defines for Ethernet MAL registers used in assembly language.
!
!  Author:      James Burke
!
!  Change Activity-
!
!  Date        Description of Change                                       BY
!  ---------   ---------------------                                       ---
!  25-May-99   Created							   JWB
!  05-May-00   New names                                                   NL
!  13-Sep-00   Port to 440GP                                               DWG
!
!-------------------------------------------------------------------------------
*/

#ifndef __MAL_INC__
#define __MAL_INC__

#ifdef __cplusplus
extern "C" {
#endif

#define MAL_CFG       0x00  /* MAL Config reg                  */
#define MAL_ESR       0x01  /* Error Status reg (Read/Clear)   */
#define MAL_IER       0x02  /* Interrupt enable reg            */
#define MAL_DBR       0x03  /* Mal Debug reg (Read only)       */
#define MAL_TXCASR    0x04  /* TX Channel active reg (set)     */
#define MAL_TXCARR    0x05  /* TX Channel active reg (Reset)   */
#define MAL_TXEOBISR  0x06  /* TX End of buffer int status reg */
#define MAL_TXDEIR    0x07  /* TX Descr. Error Int reg         */
#define MAL_TXTATTRR  0x08  /* TX PLB Attribute Register       */
#define MAL_TXBADDR   0x09  /* TX Descr. Base Addr Reg         */
#define MAL_RXCASR    0x10  /* RX Channel active reg (set)     */
#define MAL_RXCARR    0x11  /* RX Channel active reg (Reset)   */
#define MAL_RXEOBISR  0x12  /* RX End of buffer int status reg */
#define MAL_RXDEIR    0x13  /* RX Descr. Error Int reg         */
#define MAL_RXTATTRR  0x14  /* RX PLB Attribute Register       */
#define MAL_RXBADDR   0x15  /* RX Descr. Base Addr Reg         */
#define MAL_TXCTP0R   0x20  /* TX 0 Channel table pointer reg  */
#define MAL_TXCTP1R   0x21  /* TX 1 Channel table pointer reg  */
#define MAL_TXCTP2R   0x22  /* TX 2 Channel table pointer reg  */
#define MAL_TXCTP3R   0x23  /* TX 3 Channel table pointer reg  */
#define MAL_RXCTP0R   0x40  /* RX 0 Channel table pointer reg  */
#define MAL_RXCTP1R   0x41  /* RX 1 Channel table pointer reg  */
#define MAL_RCBS0     0x60  /* RX 0 Channel buffer size reg    */
#define MAL_RCBS1     0x61  /* RX 1 Channel buffer size reg    */

#define MAL0_CFG      (MAL0_DCR_BASE+MAL_CFG)      /* MAL Config reg                  */
#define MAL0_ESR      (MAL0_DCR_BASE+MAL_ESR)      /* Error Status reg (Read/Clear)   */
#define MAL0_IER      (MAL0_DCR_BASE+MAL_IER)      /* Interrupt enable reg            */
#define MAL0_DBR      (MAL0_DCR_BASE+MAL_DBR)      /* Mal Debug reg (Read only)       */
#define MAL0_TXCASR   (MAL0_DCR_BASE+MAL_TXCASR)   /* TX Channel active reg (set)     */
#define MAL0_TXCARR   (MAL0_DCR_BASE+MAL_TXCARR)   /* TX Channel active reg (Reset)   */
#define MAL0_TXEOBISR (MAL0_DCR_BASE+MAL_TXEOBISR) /* TX End of buffer int status reg */
#define MAL0_TXDEIR   (MAL0_DCR_BASE+MAL_TXDEIR)   /* TX Descr. Error Int reg         */
#define MAL0_RXCASR   (MAL0_DCR_BASE+MAL_RXCASR)   /* RX Channel active reg (set)     */
#define MAL0_RXCARR   (MAL0_DCR_BASE+MAL_RXCARR)   /* RX Channel active reg (Reset)   */
#define MAL0_RXEOBISR (MAL0_DCR_BASE+MAL_RXEOBISR) /* RX End of buffer int status reg */
#define MAL0_RXDEIR   (MAL0_DCR_BASE+MAL_RXDEIR)   /* RX Descr. Error Int reg         */
#define MAL0_TXCTP0R  (MAL0_DCR_BASE+MAL_TXCTP0R)  /* TX 0 Channel table pointer reg  */
#define MAL0_TXCTP1R  (MAL0_DCR_BASE+MAL_TXCTP1R)  /* TX 1 Channel table pointer reg  */
#define MAL0_TXCTP2R  (MAL0_DCR_BASE+MAL_TXCTP2R)  /* TX 2 Channel table pointer reg  */
#define MAL0_TXCTP3R  (MAL0_DCR_BASE+MAL_TXCTP3R)  /* TX 3 Channel table pointer reg  */
#define MAL0_RXCTP0R  (MAL0_DCR_BASE+MAL_RXCTP0R)  /* RX 0 Channel table pointer reg  */
#define MAL0_RXCTP1R  (MAL0_DCR_BASE+MAL_RXCTP1R)  /* RX 1 Channel table pointer reg  */
#define MAL0_RCBS0    (MAL0_DCR_BASE+MAL_RCBS0)    /* RX 0 Channel buffer size reg    */
#define MAL0_RCBS1    (MAL0_DCR_BASE+MAL_RCBS1)    /* RX 1 Channel buffer size reg    */

/* MADMAL transmit and receive status/control bits  */
/* for COMMAC bits, refer to the COMMAC header file */

#define MAL_TX_CTRL_READY 0x8000
#define MAL_TX_CTRL_WRAP  0x4000
#define MAL_TX_CTRL_CM    0x2000
#define MAL_TX_CTRL_LAST  0x1000
#define MAL_TX_CTRL_INTR  0x0400

#define MAL_RX_CTRL_EMPTY 0x8000
#define MAL_RX_CTRL_WRAP  0x4000
#define MAL_RX_CTRL_CM	  0x2000
#define MAL_RX_CTRL_LAST  0x1000
#define MAL_RX_CTRL_FIRST 0x0800
#define MAL_RX_CTRL_INTR  0x0400

/*-----------------------------------------------------------------------------+
| MAL Configuration Register  (MAL0_CFG) for 440GP PASS 1
+-----------------------------------------------------------------------------*/
#define MAL_CFG_SR            0x80000000       /* MAL software reset            */
#define MAL_P1_CFG_PLBP_MASK  0x00C00000       /* PLB priority                  */
#define MAL_P1_CFG_PLBP_0     0x00000000       /* lowest        */
#define MAL_P1_CFG_PLBP_1	   0x00400000 
#define MAL_P1_CFG_PLBP_2	   0x00800000
#define MAL_P1_CFG_PLBP_3	   0x00C00000       /* highest       */
#define MAL_P1_CFG_GA         0x00200000       /* Guarded active                */
#define MAL_P1_CFG_OA         0x00100000       /* Ordered active                */
#define MAL_P1_CFG_PLBLE      0x00080000       /* PLB Lock error                */
#define MAL_P1_CFG_PLBLT_MASK 0x00078000       /* PLB latency timer             */
#define MAL_P1_CFG_PLBLT_DEFAULT 0x00078000    /* ????? */	  
#define MAL_P1_CFG_PLBLT_1    0x00040000
#define MAL_P1_CFG_PLBLT_2    0x00020000
#define MAL_P1_CFG_PLBLT_3    0x00010000
#define MAL_P1_CFG_PLBLT_4    0x00008000
#define MAL_CFG_PLBB          0x00004000       /* PLB burst                     */
#define MAL_CFG_OPBBL         0x00000080       /* OPB lock enable               */
#define MAL_CFG_EOPIE         0x00000004       /* End of packet interrupt enable*/
#define MAL_CFG_LEA           0x00000002       /* Lock error active             */
#define MAL_CFG_SD            0x00000001       /* Scroll descriptor             */

/*-----------------------------------------------------------------------------+
| MAL Configuration Register  (MAL0_CFG) for 440GP PASS 2
+-----------------------------------------------------------------------------*/
#define MAL_CFG_SR            0x80000000       /* MAL software reset            */
#define MAL_P2_CFG_RPP_MASK   0x00C00000       /* Read PLB Priority             */
#define MAL_P2_CFG_RPP_0      0x00000000       /* lowest        */
#define MAL_P2_CFG_RPP_1         0x00400000 
#define MAL_P2_CFG_RPP_2         0x00800000
#define MAL_P2_CFG_RPP_3         0x00C00000    /* highest       */
#define MAL_P2_CFG_RMBS_MASK  0x00300000       /* Read Max Burst Size           */
#define MAL_P2_CFG_RMBS_4        0x00000000    /* Max burst size of 4  */
#define MAL_P2_CFG_RMBS_8        0x00100000    /* Max burst size of 8  */ 
#define MAL_P2_CFG_RMBS_16       0x00200000    /* Max burst size of 16 */
#define MAL_P2_CFG_RMBS_32       0x00300000    /* Max burst size of 32 */
#define MAL_P2_CFG_WPP_MASK   0x000C0000       /* Write PLB priority            */
#define MAL_P2_CFG_WPP_0         0x00000000    /* lowest        */
#define MAL_P2_CFG_WPP_1         0x00040000 
#define MAL_P2_CFG_WPP_2         0x00080000
#define MAL_P2_CFG_WPP_3         0x000C0000    /* highest       */
#define MAL_P2_CFG_WMBS_MASK  0x00030000       /* Write Max Burst Size          */
#define MAL_P2_CFG_WMBS_4        0x00000000    /* Max burst size of 4  */
#define MAL_P2_CFG_WMBS_8        0x00010000    /* Max burst size of 8  */
#define MAL_P2_CFG_WMBS_16       0x00020000    /* Max burst size of 16 */
#define MAL_P2_CFG_WMBS_32       0x00030000    /* Max burst size of 32 */
#define MAL_P2_CFG_PLBLE      0x00008000       /* PLB Lock error                */
#define MAL_CFG_PLBB          0x00004000       /* PLB burst                     */
#define MAL_CFG_OPBBL         0x00000080       /* OPB lock enable               */
#define MAL_CFG_EOPIE         0x00000004       /* End of packet interrupt enable*/
#define MAL_CFG_LEA           0x00000002       /* Lock error active             */
#define MAL_CFG_SD            0x00000001       /* Scroll descriptor             */

/*-----------------------------------------------------------------------------+
| MAL Error Status Register  (MAL0_ESR)
|    The 1st group of bits is for the "error status" part of the register
|       (locked or last occuring)
|    The 2nd group of bits is for the "interrupt status" part of the register
+-----------------------------------------------------------------------------*/
#define MAL_ESR_EVB       0x80000000       /* Is there a valid error?        */
#define MAL_ESR_CIDT      0x40000000       /* Is Channel ID an RX channel?   */
#define MAL_ESR_CID_MASK  0x3E000000       /* Channel ID mask                */
#define MAL_ESR_PTE       0x00800000       /* PLB Timeout Error              */
#define MAL_ESR_PRE       0x00400000       /* PLB Read Error                 */
#define MAL_ESR_PWE       0x00200000       /* PLB Write Error                */
#define MAL_ESR_DE        0x00100000       /* TX descriptor (not first) error*/
#define MAL_ESR_ONE       0x00080000       /* OPB non-fullword error         */
#define MAL_ESR_OTE       0x00040000       /* OPB timeout error              */
#define MAL_ESR_OSE       0x00020000       /* OPB slave error                */
#define MAL_ESR_PEIN      0x00010000       /* PLB error                      */
#define MAL_ESR_PTEI      0x00000080       /* PLB Timeout Error Interrupt    */
#define MAL_ESR_PREI      0x00000040       /* PLB Read Error Interrupt       */
#define MAL_ESR_PWEI      0x00000020       /* PLB Write Error Interrupt      */
#define MAL_ESR_DEI       0x00000010       /* TX descriptor (not first) error*/
#define MAL_ESR_ONEI      0x00000008       /* OPB non-fullword error         */
#define MAL_ESR_OTEI      0x00000004       /* OPB timeout error              */
#define MAL_ESR_OSEI      0x00000002       /* OPB slave error                */
#define MAL_ESR_PBEI      0x00000001       /* PLB error                      */

/*-----------------------------------------------------------------------------+
| MAL Interrupt Enable Register  (MAL0_IER)
|   These bits enable the events that will cause the activation of
|   EXT_IRQ_MAL_SERR in the Universal Interrupt Controller.
+-----------------------------------------------------------------------------*/
#define MAL_P1_IER_DE     0x00000010       /* TX descriptor (not first) error*/
#define MAL_P1_IER_NWE    0x00000008       /* OPB non-fullword error         */
#define MAL_P1_IER_TO     0x00000004       /* OPB timeout error              */
#define MAL_P1_IER_OPB    0x00000002       /* OPB slave error                */
#define MAL_P1_IER_PLB    0x00000001       /* PLB error                      */

#define MAL_P2_IER_PT     0x00000080       /* PLB Timeout Interrupt          */
#define MAL_P2_IER_PRE    0x00000040       /* PLB Read Error Interrupt       */
#define MAL_P2_IER_PWE    0x00000020       /* PLB Write Error Interrupt      */
#define MAL_P2_IER_DE     0x00000010       /* Descriptor Error Interrupt     */
#define MAL_P2_IER_OTE    0x00000004       /* OPB Timeout Error Interrupt    */
#define MAL_P2_IER_OE  	  0x00000002       /* OPB Error Interrupt            */
#define MAL_P2_IER_PE     0x00000001       /* PLB Error Interrupt            */

#define MAL_CHANNEL0      0x80000000
#define MAL_CHANNEL1      0x40000000
#define MAL_CHANNEL2      0x20000000
#define MAL_CHANNEL3      0x10000000
#define MAL_CHANNEL4      0x08000000
#define MAL_CHANNEL5      0x04000000
#define MAL_CHANNEL6      0x02000000
#define MAL_CHANNEL7      0x01000000
#define MAL_CHANNEL8      0x00800000
#define MAL_CHANNEL9      0x00400000
#define MAL_CHANNEL10     0x00200000
#define MAL_CHANNEL11     0x00100000
#define MAL_CHANNEL12     0x00080000
#define MAL_CHANNEL13     0x00040000
#define MAL_CHANNEL14     0x00020000
#define MAL_CHANNEL15     0x00010000
#define MAL_CHANNEL16     0x00008000
#define MAL_CHANNEL17     0x00004000
#define MAL_CHANNEL18     0x00002000
#define MAL_CHANNEL19     0x00001000
#define MAL_CHANNEL20     0x00000800
#define MAL_CHANNEL21     0x00000400
#define MAL_CHANNEL22     0x00000200
#define MAL_CHANNEL23     0x00000100
#define MAL_CHANNEL24     0x00000080
#define MAL_CHANNEL25     0x00000040
#define MAL_CHANNEL26     0x00000020
#define MAL_CHANNEL27     0x00000010
#define MAL_CHANNEL28     0x00000008
#define MAL_CHANNEL29     0x00000004
#define MAL_CHANNEL30     0x00000002
#define MAL_CHANNEL31     0x00000001

#define MAL_MAXDESCRIPTORS 256
#define MAL_MAXTXBLOCK     4095
#define MAL_MAXRXBLOCK     4080

#define MAL_BUFF_ALIGNMENT_MASK 0xFFFFFFE0  /* 32 byte buffer alignment */

#ifdef __cplusplus
}
#endif

#endif /* __MAL_INC__ */
