/* m4/sram.inc, openbios_ebony, ebony_bios 7/9/00 14:37:54 
!------------------------------------------------------------------------------+
!
!       This source code has been made available to you by IBM on an AS-IS
!       basis.  Anyone receiving this source is licensed under IBM
!       copyrights to use it in any way he or she deems fit, including
!       copying it, modifying it, compiling it, and redistributing it either
!       with or without modifications.  No license under IBM patents or
!       patent applications is to be implied by the copyright license.
!
!       Any user of this software should understand that IBM cannot provide
!       technical support for this software and will not be responsible for
!       any consequences resulting from the use of this software.
!
!       Any person who transfers this source code or any derivative work
!       must include the IBM copyright notice, this paragraph, and the
!       preceding two paragraphs in the transferred software.
!
!       COPYRIGHT   I B M   CORPORATION 1995
!       LICENSED MATERIAL  -  PROGRAM PROPERTY OF I B M
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
!  File Name:   sram.inc
!
!  Function:    Defines for SRAM registers used in assembly language.
!
!  Author:      Dan Goehring
!
!  Change Activity-
!
!  Date        Description of Change                                       BY
!  ---------   ---------------------                                       ---
!  31-Jan-01   Created                                                     DWG
!
!-------------------------------------------------------------------------------
*/

#ifndef __SRAM_INC__
#define __SRAM_INC__

#ifdef __cplusplus
extern "C" {
#endif

#define SRAM_BASE_PLB_UA 0x0
#define SRAM_BASE_PLB 0x80000000

#define SRAM_SB0CR   0x00    /* SRAM Bank 0 Configuration Register        */
#define SRAM_SB1CR   0x01    /* SRAM Bank 1 Configuration Register        */
#define SRAM_SB2CR   0x02    /* SRAM Bank 2 Configuration Register        */
#define SRAM_SB3CR   0x03    /* SRAM Bank 3 Configuration Register        */
#define SRAM_BEAR    0x04    /* SRAM Bus Error Address Register           */
#define SRAM_BESR0   0x05    /* Bus Error Status Register 0               */
#define SRAM_BESR1   0x06    /* Bus Error Status Register 1               */
#define SRAM_PMEG    0x07    /* Power Management                          */
#define SRAM_DPC     0x0A    /* Data Parity Checking Register             */

#define SRAM0_SB0CR   (SRAM0_DCR_BASE + SRAM_SB0CR)
#define SRAM0_SB1CR   (SRAM0_DCR_BASE + SRAM_SB1CR)
#define SRAM0_SB2CR   (SRAM0_DCR_BASE + SRAM_SB2CR)
#define SRAM0_SB3CR   (SRAM0_DCR_BASE + SRAM_SB3CR)
#define SRAM0_BEAR    (SRAM0_DCR_BASE + SRAM_BEAR)
#define SRAM0_BESR0   (SRAM0_DCR_BASE + SRAM_BESR0)
#define SRAM0_BESR1   (SRAM0_DCR_BASE + SRAM_BESR1)
#define SRAM0_PMEG    (SRAM0_DCR_BASE + SRAM_PMEG)
#define SRAM0_DPC     (SRAM0_DCR_BASE + SRAM_DPC)

/*------------- SRAM BESR0 REG defines --------------------------------------*/
#define SRAM_BESR0_ET0_MASK      0xE0000000
#define SRAM_BESR0_RWS0          0x10000000
#define SRAM_BESR0_FL0           0x08000000
#define SRAM_BESR0_AL0           0x04000000
#define SRAM_BESR0_ET1_MASK      0x03800000
#define SRAM_BESR0_RWS1          0x00400000
#define SRAM_BESR0_FL1           0x00200000
#define SRAM_BESR0_AL1           0x00100000
#define SRAM_BESR0_ET2_MASK      0x000E0000
#define SRAM_BESR0_RWS2          0x00010000
#define SRAM_BESR0_FL2           0x00008000
#define SRAM_BESR0_AL2           0x00004000
#define SRAM_BESR0_ET3_MASK      0x00003800
#define SRAM_BESR0_RWS3          0x00000400
#define SRAM_BESR0_FL3           0x00000200
#define SRAM_BESR0_AL3           0x00000100

/*------------- SRAM BESR1 REG defines --------------------------------------*/
#define SRAM_BESR1_ET5_MASK      0x03800000
#define SRAM_BESR1_RWS5          0x00400000
#define SRAM_BESR1_FL5           0x00200000
#define SRAM_BESR1_AL5           0x00100000
#define SRAM_BESR1_ET6_MASK      0x000E0000
#define SRAM_BESR1_RWS6          0x00010000
#define SRAM_BESR1_FL6           0x00008000
#define SRAM_BESR1_AL6           0x00004000
#define SRAM_BESR1_ET7_MASK      0x00003800
#define SRAM_BESR1_RWS7          0x00000400
#define SRAM_BESR1_FL7           0x00000200
#define SRAM_BESR1_AL7           0x00000100

/*------------- SRAM PMEG REG defines --------------------------------------*/
#define SRAM_PMEG_PM_EN          0x80000000
#define SRAM_PMEG_PM_CNT_MASK    0x7E000000
#define SRAM_PMEG_PM_DFLT_MASK   0x01E00000

/*------------- SRAM SBXCR REG defines --------------------------------------*/
#define SRAM_SBXCR_BAS_MASK      0xFFFFF000
#define SRAM_SBXCR_BAS_ENCODE(x) ((x)&0xFFFFF000)
#define SRAM_SBXCR_BAS_DECODE(x) ((x)&0xFFFFF000)
#define SRAM_SBXCR_BS_MASK       0x00000E00
#define   SRAM_SBXCR_BS_4KB      0x00000000
#define   SRAM_SBXCR_BS_8KB      0x00000200
#define SRAM_SBXCR_BU_MASK       0x00000180
#define   SRAM_SBXCR_BU_DIS      0x00000000
#define   SRAM_SBXCR_BU_RO       0x00000080
#define   SRAM_SBXCR_BU_RW       0x00000180
#define SRAM_SBXCR_UA_MASK       0x0000000F
#define SRAM_SBXCR_UA_ENCODE(x) (((x)&0xF)<<0)
#define SRAM_SBXCR_UA_DECODE(x) (((x)>>0)&0xF)

#ifdef __cplusplus
}
#endif

#endif /* __SRAM_INC__ */
