/*
 * (C) Copyright 2000-2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <osak.h>
#include "zlib.h"
#include <stdio.h>
#include <string.h>
#include <errno.h>
//#include <ctype.h>
#include "fw_env.h"
#include <malloc.h>

typedef unsigned char uchar;

#define	CMD_GETENV	"fw_printenv"
#define	CMD_SETENV	"fw_setenv"

typedef struct envdev_s {
	uchar devname[16];		/* Device name */
	ulong devoff;			/* Device offset */
	ulong env_size;			/* environment size */
	ulong erase_size;		/* device erase size */
} envdev_t;

//static envdev_t envdevices[2];
static int curdev;

//#define DEVNAME(i)    envdevices[(i)].devname
//#define DEVOFFSET(i)  envdevices[(i)].devoff
//#define ENVSIZE(i)    envdevices[(i)].env_size
//#define DEVESIZE(i)   envdevices[(i)].erase_size

//#define CFG_ENV_SIZE ENVSIZE(curdev)

#define ENV_SIZE      32768
#define CFG_ENV_BDK_PARTITION 0
#define	CFG_ENV_SIZE		       32768
#define BDK_IMG_SIGNATURE "BIPO"        // default signature of BDK partition that contains the registry
#define BDK_SIGNATURE_NAME_LEN         4
#define BDK_SIGN_OFFSET                8
#define PENV_IMG_SIGNATURE "PENV"       // PPC Environment sub-partition signature
#define PENV_SIGNATURE_NAME_LEN         4
#define PENV_SIGN_OFFSET                8

#define REGISTRY_IMG_START_BLOCK (0)   // first block to store registry
#define REGISTRY_IMG_LENGTH CFG_ENV_SIZE //size of registry in bytes
#define START_ADD_REG_1 REGISTRY_IMG_START_BLOCK   
#define START_ADD_REG_2 (START_ADD_REG_1 + (REGISTRY_IMG_LENGTH >> bdkVol->erasableBlockBits) )

typedef struct environment_s {
	ulong crc;			/* CRC32 over data bytes    */
	uchar flags;			/* active or obsolete */
	uchar *data;
} env_t;

static env_t environment;

static int HaveRedundEnv = 0;

static uchar active_flag = 1;
static uchar obsolete_flag = 0;


#define XMK_STR(x)	#x
#define MK_STR(x)	XMK_STR(x)

static uchar default_environment[] = {
#if defined(CONFIG_BOOTARGS)
	"bootargs=" CONFIG_BOOTARGS "\0"
#endif
#if defined(CONFIG_BOOTCOMMAND)
	"bootcmd=" CONFIG_BOOTCOMMAND "\0"
#endif
#if defined(CONFIG_RAMBOOTCOMMAND)
	"ramboot=" CONFIG_RAMBOOTCOMMAND "\0"
#endif
#if defined(CONFIG_NFSBOOTCOMMAND)
	"nfsboot=" CONFIG_NFSBOOTCOMMAND "\0"
#endif
#if defined(CONFIG_BOOTDELAY) && (CONFIG_BOOTDELAY >= 0)
	"bootdelay=" MK_STR (CONFIG_BOOTDELAY) "\0"
#endif
#if defined(CONFIG_BAUDRATE) && (CONFIG_BAUDRATE >= 0)
	"baudrate=" MK_STR (CONFIG_BAUDRATE) "\0"
#endif
#ifdef	CONFIG_LOADS_ECHO
	"loads_echo=" MK_STR (CONFIG_LOADS_ECHO) "\0"
#endif
#ifdef	CONFIG_ETHADDR
	"ethaddr=" MK_STR (CONFIG_ETHADDR) "\0"
#endif
#ifdef	CONFIG_ETH1ADDR
	"eth1addr=" MK_STR (CONFIG_ETH1ADDR) "\0"
#endif
#ifdef	CONFIG_ETH2ADDR
	"eth2addr=" MK_STR (CONFIG_ETH2ADDR) "\0"
#endif
#ifdef	CONFIG_ETHPRIME
	"ethprime=" CONFIG_ETHPRIME "\0"
#endif
#ifdef	CONFIG_IPADDR
	"ipaddr=" MK_STR (CONFIG_IPADDR) "\0"
#endif
#ifdef	CONFIG_SERVERIP
	"serverip=" MK_STR (CONFIG_SERVERIP) "\0"
#endif
#ifdef	CFG_AUTOLOAD
	"autoload=" CFG_AUTOLOAD "\0"
#endif
#ifdef	CONFIG_ROOTPATH
	"rootpath=" MK_STR (CONFIG_ROOTPATH) "\0"
#endif
#ifdef	CONFIG_GATEWAYIP
	"gatewayip=" MK_STR (CONFIG_GATEWAYIP) "\0"
#endif
#ifdef	CONFIG_NETMASK
	"netmask=" MK_STR (CONFIG_NETMASK) "\0"
#endif
#ifdef	CONFIG_HOSTNAME
	"hostname=" MK_STR (CONFIG_HOSTNAME) "\0"
#endif
#ifdef	CONFIG_BOOTFILE
	"bootfile=" MK_STR (CONFIG_BOOTFILE) "\0"
#endif
#ifdef	CONFIG_LOADADDR
	"loadaddr=" MK_STR (CONFIG_LOADADDR) "\0"
#endif
#ifdef	CONFIG_PREBOOT
	"preboot=" CONFIG_PREBOOT "\0"
#endif
#ifdef	CONFIG_CLOCKS_IN_MHZ
	"clocks_in_mhz=" "1" "\0"
#endif
#if defined(CONFIG_PCI_BOOTDELAY) && (CONFIG_PCI_BOOTDELAY > 0)
	"pcidelay=" MK_STR (CONFIG_PCI_BOOTDELAY) "\0"
#endif
#ifdef  CONFIG_EXTRA_ENV_SETTINGS
	CONFIG_EXTRA_ENV_SETTINGS
#endif
	"\0"			/* Termimate env_t data with 2 NULs */
};

#define CFG_ENV_BDK_PARTITION 0


int mdocp_env_init(void);
int mdocp_env_write(uchar *ptr, int size);
static uchar *envmatch (uchar * s1, uchar * s2);
static int env_init (void);
static int parse_config (void);
//static void _print_data(unsigned char *buf,int len);

uLong crc32(uLong crc, const Bytef *buf, uInt len);

static inline ulong getenvsize (void)
{
	ulong rc = CFG_ENV_SIZE - sizeof (long);

	if (HaveRedundEnv)
		rc -= sizeof (char);
	return rc;
}

/*
 * Search the environment for a variable.
 * Return the value, if found, or NULL, if not found.
 */
unsigned char *fw_getenv (unsigned char *name)
{
	uchar *env, *nxt;

	if (env_init ())
		return (NULL);

	for (env = environment.data; *env; env = nxt + 1) {
		uchar *val;

		for (nxt = env; *nxt; ++nxt) {
			if (nxt >= &environment.data[ENV_SIZE]) {
				fprintf (stderr, "## Error: "
					"environment not terminated\n");
				return (NULL);
			}
		}
		val = envmatch (name, env);
		if (!val)
			continue;
		return (val);
	}
	return (NULL);
}

/*
 * Print the current definition of one, or more, or all
 * environment variables
 */
void fw_printenv (int argc, char *argv[])
{
	uchar *env, *nxt;
	int i, n_flag;

	if (env_init ())
		return;

	if (argc == 1) {		/* Print all env variables  */
		for (env = environment.data; *env; env = nxt + 1) {
			for (nxt = env; *nxt; ++nxt) {
				if (nxt >= &environment.data[ENV_SIZE]) {
					fprintf (stderr, "## Error: "
						"environment not terminated\n");
					return;
				}
			}

			printf ("%s\n", env);
		}
		return;
	}

	if (strcmp (argv[1], "-n") == 0) {
		n_flag = 1;
		++argv;
		--argc;
		if (argc != 2) {
			fprintf (stderr, "## Error: "
				"`-n' option requires exactly one argument\n");
			return;
		}
	} else {
		n_flag = 0;
	}

	for (i = 1; i < argc; ++i) {	/* print single env variables   */
		uchar *name = argv[i];
		uchar *val = NULL;

		for (env = environment.data; *env; env = nxt + 1) {

			for (nxt = env; *nxt; ++nxt) {
				if (nxt >= &environment.data[ENV_SIZE]) {
					fprintf (stderr, "## Error: "
						"environment not terminated\n");
					return;
				}
			}
			val = envmatch (name, env);
			if (val) {
				if (!n_flag) {
					fputs (name, stdout);
					putc ('=', stdout);
				}
				puts (val);
				break;
			}
		}
		if (!val)
			fprintf (stderr, "## Error: \"%s\" not defined\n", name);
	}
}

/*
 * Deletes or sets environment variables. Returns errno style error codes:
 * 0	  - OK
 * EINVAL - need at least 1 argument
 * EROFS  - certain variables ("ethaddr", "serial#") cannot be
 *	    modified or deleted
 *
 */
int fw_setenv (int argc, char *argv[])
{
	int i, len;
	uchar *env, *nxt;
	uchar *oldval = NULL;
	uchar *name;

	if (argc < 2) {
		return (EINVAL);
	}

	if (env_init ())
		return (errno);

	name = argv[1];

	/*
	 * search if variable with this name already exists
	 */
	for (nxt = env = environment.data; *env; env = nxt + 1) {
		for (nxt = env; *nxt; ++nxt) {
			if (nxt >= &environment.data[ENV_SIZE]) {
				fprintf (stderr, "## Error: "
					"environment not terminated\n");
				return (EINVAL);
			}
		}
		if ((oldval = envmatch (name, env)) != NULL)
			break;
	}

	/*
	 * Delete any existing definition
	 */
	if (oldval) {
		/*
		 * Ethernet Address and serial# can be set only once
		 */
		if ((strcmp (name, "ethaddr") == 0) ||
			(strcmp (name, "serial#") == 0)) {
			fprintf (stderr, "Can't overwrite \"%s\"\n", name);
			return (EROFS);
		}

		if (*++nxt == '\0') {
			*env = '\0';
		} else {
			for (;;) {
				*env = *nxt++;
				if ((*env == '\0') && (*nxt == '\0'))
					break;
				++env;
			}
		}
		*++env = '\0';
	}

	/* Delete only ? */
	if (argc < 3)
		goto WRITE_FLASH;

	/*
	 * Append new definition at the end
	 */
	for (env = environment.data; *env || *(env + 1); ++env);
	if (env > environment.data)
		++env;
	/*
	 * Overflow when:
	 * "name" + "=" + "val" +"\0\0"  > CFG_ENV_SIZE - (env-environment)
	 */
	len = strlen (name) + 2;
	/* add '=' for first arg, ' ' for all others */
	for (i = 2; i < argc; ++i) {
		len += strlen (argv[i]) + 1;
	}
	if (len > (&environment.data[ENV_SIZE] - env)) {
		fprintf (stderr,
			"Error: environment overflow, \"%s\" deleted\n",
			name);
		return (-1);
	}
	while ((*env = *name++) != '\0')
		env++;
	for (i = 2; i < argc; ++i) {
		uchar *val = argv[i];

		*env = (i == 2) ? '=' : ' ';
		while ((*++env = *val++) != '\0');
	}

	/* end is marked with double '\0' */
	*++env = '\0';

  WRITE_FLASH:

	/* Update CRC */
	environment.crc = crc32 (0, environment.data, ENV_SIZE - sizeof(ulong));
  environment.data -= sizeof(long);
  *(ulong *)(environment.data) = environment.crc;

	/* write environment back to flash */
  if (mdocp_env_write(environment.data, ENV_SIZE)) {
   	fprintf (stderr, "Error: can't write fw_env to flash\n");
		return (-1);
	}

	return (0);
}

/*
 * s1 is either a simple 'name', or a 'name=value' pair.
 * s2 is a 'name=value' pair.
 * If the names match, return the value of s2, else NULL.
 */

static uchar *envmatch (uchar * s1, uchar * s2)
{

	while (*s1 == *s2++)
		if (*s1++ == '=')
			return (s2);
	if (*s1 == '\0' && *(s2 - 1) == '=')
		return (s2);
	return (NULL);
}

static PhysicalInfo physicalInfo;
static IOreq ioreq={0,0,NULL,NULL,0,0};
IOreq*SetIoreq(unsigned int irHandle,unsigned irFlags,void FAR1*irData,long irLength,long irCount)
{
	ioreq.irHandle=irHandle;
	ioreq.irFlags=irFlags;
	ioreq.irData=irData;
	ioreq.irLength=irLength;
	ioreq.irCount=irCount;
	return &ioreq;
}

/*
 * Prevent confusion if running from erased flash memory
 */
static int env_init (void)
{
	int crc1, crc1_ok;
	uchar *addr1;

	int crc2, crc2_ok;
	uchar flag1, flag2, *addr2;
  
  if (flGetPhysicalInfo(SetIoreq(0,0,&physicalInfo,0,0)) != flOK)
  {
		fprintf (stderr,
			"Cannot get the physical info\n");
		return 1;
  }
//	printf("%s found in 0x%lx.\n%dM media, %dK unit\n",TransDocType(physicalInfo.mediaType),ioreq.irLength,(int)(physicalInfo.mediaSize>>20),(int)(physicalInfo.unitSize/1024));

	if (parse_config ())
		return 1;

	if ((addr1 = calloc (1, ENV_SIZE)) == NULL) {
		fprintf (stderr,
			"Not enough memory for environment (%d bytes)\n",
			ENV_SIZE);
		return (errno);
	}

	/* read environment from FLASH to local buffer */
	environment.data = addr1;
	curdev = 0;
  if (mdocp_env_init())
  {
			fprintf (stderr,
				"Error reading the environemnt from the DOC device\n");
			free (addr1);
      return 1;
  }
//  _print_data(addr1, ENV_SIZE);

	crc1_ok = ((crc1 = crc32 (0, environment.data, ENV_SIZE - sizeof(ulong)))
			   == environment.crc);
	if (!HaveRedundEnv) {
		if (!crc1_ok) {
			fprintf (stderr,
				"Warning: Bad CRC, using default environment\n");
			environment.data = default_environment;
			free (addr1);
		}
	} else {
		flag1 = environment.flags;

		curdev = 1;
		if ((addr2 = calloc (1, ENV_SIZE)) == NULL) {
			fprintf (stderr,
				"Not enough memory for environment (%d bytes)\n",
				ENV_SIZE);
			return (errno);
		}
		environment.data = addr2;

		crc2_ok = ((crc2 = crc32 (0, environment.data, ENV_SIZE - sizeof(ulong)))
				   == environment.crc);
		flag2 = environment.flags;

		if (crc1_ok && !crc2_ok) {
			environment.data = addr1;
			environment.flags = flag1;
			environment.crc = crc1;
			curdev = 0;
			free (addr2);
		} else if (!crc1_ok && crc2_ok) {
			environment.data = addr2;
			environment.flags = flag2;
			environment.crc = crc2;
			curdev = 1;
			free (addr1);
		} else if (!crc1_ok && !crc2_ok) {
			fprintf (stderr,
				"Warning: Bad CRC, using default environment\n");
			environment.data = default_environment;
			curdev = 0;
			free (addr2);
			free (addr1);
		} else if (flag1 == active_flag && flag2 == obsolete_flag) {
			environment.data = addr1;
			environment.flags = flag1;
			environment.crc = crc1;
			curdev = 0;
			free (addr2);
		} else if (flag1 == obsolete_flag && flag2 == active_flag) {
			environment.data = addr2;
			environment.flags = flag2;
			environment.crc = crc2;
			curdev = 1;
			free (addr1);
		} else if (flag1 == flag2) {
			environment.data = addr1;
			environment.flags = flag1;
			environment.crc = crc1;
			curdev = 0;
			free (addr2);
		} else if (flag1 == 0xFF) {
			environment.data = addr1;
			environment.flags = flag1;
			environment.crc = crc1;
			curdev = 0;
			free (addr2);
		} else if (flag2 == 0xFF) {
			environment.data = addr2;
			environment.flags = flag2;
			environment.crc = crc2;
			curdev = 1;
			free (addr1);
		}
	}
	return (0);
}

static int parse_config ()
{
	return 0;
}

int
mdocp_env_init(void)
{
  BDKStruct bdkStruct;

  /*
   * Read the primary registry record
   */
  bdkStruct.startingBlock = START_ADD_REG_1;
  bdkStruct.length        = REGISTRY_IMG_LENGTH;
  memcpy(bdkStruct.oldSign, PENV_IMG_SIGNATURE, PENV_SIGNATURE_NAME_LEN);
  bdkStruct.flags         = EDC;
  bdkStruct.signOffset    = PENV_SIGN_OFFSET;
  ioreq.irHandle          = CFG_ENV_BDK_PARTITION << 4;
  ioreq.irData            = &bdkStruct;

  if (bdkReadInit(&ioreq) != flOK)
  {
    return -1;
  }

  bdkStruct.startingBlock = START_ADD_REG_1;
  bdkStruct.length        = REGISTRY_IMG_LENGTH;
  bdkStruct.bdkBuffer     = (byte FAR1 *)(environment.data);
  ioreq.irHandle          = CFG_ENV_BDK_PARTITION << 4;
  ioreq.irData            = &bdkStruct;

  if (bdkReadBlock(&ioreq) != flOK)
  {
    return -1;
  }
  environment.crc = *(ulong *)(environment.data);
  environment.data = (unsigned char *)(environment.data + sizeof(long));

  return 0;
} /* mdocp_env_init */

int
mdocp_env_write(uchar *ptr, int size)
{
  IOreq ioreq = {0,0,NULL,NULL,0,0};
  BDKStruct bdkStruct;
	
  /*
   * Update the primary registry record
   */
  bdkStruct.startingBlock = START_ADD_REG_1;
  bdkStruct.length        = size;
  memcpy(bdkStruct.oldSign, PENV_IMG_SIGNATURE, PENV_SIGNATURE_NAME_LEN);
  bdkStruct.flags         = BDK_COMPLETE_IMAGE_UPDATE|EDC;
  bdkStruct.signOffset    = PENV_SIGN_OFFSET;
  ioreq.irHandle          = CFG_ENV_BDK_PARTITION << 4;
  ioreq.irData            = &bdkStruct;

  if (bdkWriteInit(&ioreq) != flOK)
    return 1;

  bdkStruct.bdkBuffer     = (byte FAR1 *)ptr;
  bdkStruct.flags         = ERASE_BEFORE_WRITE;
  ioreq.irHandle          = CFG_ENV_BDK_PARTITION << 4;
  ioreq.irData            = &bdkStruct;

  if (bdkWriteBlock(&ioreq) != flOK)
    return 1;

  return 0;

} /* mdocp_env_write */

uint GetDocWinLAddress(void)
{
        return 0xfff00000;
}

uint GetDocWinHAddress(void)
{
        return 0xfff00000;
}

/* Macros for min/max.  */
#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

#if 0
/*
 * Few little utilities borrowed from the tcpdump
 */
static
void _print_asc(unsigned char *buf,int len)
{
  int i;
  for (i=0;i<len;i++)
    printf("%c",isprint(buf[i])?buf[i]:'.');
}

static
void _print_data(unsigned char *buf,int len)
{
  int i=0;
  if (len<=0) return;
  printf("[%03X] ",i);
  for (i=0;i<len;) {
    printf("%02X ",(int)buf[i]);
    i++;
    if (i%16 == 0) {      
      _print_asc(&buf[i-16],16);
      if (i<len) printf("\n[%03X] ",i);
    }
  }
  if (i%16) {
    int n;

    n = 16 - (i%16);
    printf(" ");
    while (n--) printf("   ");

    n = MIN(8,i%16);
    _print_asc(&buf[i-(i%16)],n);
    n = (i%16) - n;
    if (n>0) _print_asc(&buf[i-n],n); 
  }
  printf("\n");
  fflush(stdout);
}
#endif

